@@ -653,7 +653,9 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
             'full_name_format',
             're_expression',
             'cached_auth_timeout',
-            'auto_private_groups']
+            'auto_private_groups',
+            'pam_gssapi_services',
+            'pam_gssapi_check_upn']
 
         self.assertTrue(type(options) == dict,
                         "Options should be a dictionary")
@@ -736,6 +738,7 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
              'krb5_fast_principal',
              'krb5_canonicalize',
              'krb5_use_enterprise_principal',
+             'krb5_use_subdomain_realm',
              'krb5_use_kdcinfo',
              'krb5_map_user'])
 
@@ -899,6 +902,7 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
             'krb5_fast_principal',
             'krb5_canonicalize',
             'krb5_use_enterprise_principal',
+            'krb5_use_subdomain_realm',
             'krb5_use_kdcinfo',
             'krb5_map_user']
 
@@ -1030,7 +1034,9 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
             'full_name_format',
             're_expression',
             'cached_auth_timeout',
-            'auto_private_groups']
+            'auto_private_groups',
+            'pam_gssapi_services',
+            'pam_gssapi_check_upn']
 
         self.assertTrue(type(options) == dict,
                         "Options should be a dictionary")
@@ -1114,6 +1120,7 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
              'krb5_fast_principal',
              'krb5_canonicalize',
              'krb5_use_enterprise_principal',
+             'krb5_use_subdomain_realm',
              'krb5_use_kdcinfo',
              'krb5_map_user'])
 
