@@ -566,6 +566,12 @@ static errno_t sysdb_domain_cache_upgrade(TALLOC_CTX *mem_ctx,
         }
     }
 
+    if (strcmp(version, SYSDB_VERSION_0_21) == 0) {
+        ret = sysdb_upgrade_21(sysdb, &version);
+        if (ret != EOK) {
+            goto done;
+        }
+    }
 
     ret = EOK;
 done:
