@@ -226,7 +226,7 @@ static char *get_issuer_subject_str(TALLOC_CTX *mem_ctx, X509 *cert)
 
     bio_mem = BIO_new(BIO_s_mem());
     if (bio_mem == NULL) {
-        DEBUG(SSSDBG_OP_FAILURE, "BIO_new failed.\n");
+        DEBUG(SSSDBG_CRIT_FAILURE, "BIO_new failed.\n");
         return NULL;
     }
 
@@ -372,8 +372,8 @@ static errno_t do_ocsp(struct p11_ctx *p11_ctx, X509 *cert)
     ocsp_dgst = get_dgst(p11_ctx->cert_verify_opts->ocsp_dgst);
     if (ocsp_dgst == NULL) {
         DEBUG(SSSDBG_OP_FAILURE, "Cannot determine configured digest function "
-                                 "for OCSP, using default sha256.\n");
-        ocsp_dgst = EVP_sha256();
+                                 "for OCSP, using default sha1.\n");
+        ocsp_dgst = EVP_sha1();
     }
     cid = OCSP_cert_to_id(ocsp_dgst, cert, issuer);
     if (cid == NULL) {
@@ -591,7 +591,7 @@ errno_t init_p11_ctx(TALLOC_CTX *mem_ctx, const char *ca_db,
     ret = SSL_library_init();
 #endif
     if (ret != 1) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "Failed to initialize OpenSSL.\n");
+        DEBUG(SSSDBG_FATAL_FAILURE, "Failed to initialize OpenSSL.\n");
         return EIO;
     }
 
@@ -1293,7 +1293,7 @@ static CK_RV get_preferred_rsa_mechanism(TALLOC_CTX *mem_ctx,
         { CKM_SHA256_RSA_PKCS, "CKM_SHA256_RSA_PKCS", EVP_sha256(), "sha256" },
         { CKM_SHA224_RSA_PKCS, "CKM_SHA224_RSA_PKCS", EVP_sha224(), "sha224" },
         { CKM_SHA1_RSA_PKCS,   "CKM_SHA1_RSA_PKCS",   EVP_sha1(),   "sha1" },
-        { 0, NULL }
+        { 0, NULL, NULL, NULL }
     };
 
     *preferred_mechanism = CKM_SHA1_RSA_PKCS;
@@ -1546,35 +1546,38 @@ static errno_t wait_for_card(CK_FUNCTION_LIST *module, CK_SLOT_ID *slot_id)
     CK_RV rv;
     CK_SLOT_INFO info;
 
-    rv = module->C_WaitForSlotEvent(wait_flags, slot_id, NULL);
-    if (rv != CKR_OK) {
-        if (rv != CKR_FUNCTION_NOT_SUPPORTED) {
+    do {
+        rv = module->C_WaitForSlotEvent(wait_flags, slot_id, NULL);
+        if (rv != CKR_OK && rv != CKR_FUNCTION_NOT_SUPPORTED) {
             DEBUG(SSSDBG_OP_FAILURE,
                   "C_WaitForSlotEvent failed [%lu][%s].\n",
                   rv, p11_kit_strerror(rv));
             return EIO;
         }
 
-        /* Poor man's wait */
-        do {
+        if (rv == CKR_FUNCTION_NOT_SUPPORTED) {
+            /* Poor man's wait */
             sleep(10);
-            rv = module->C_GetSlotInfo(*slot_id, &info);
-            if (rv != CKR_OK) {
-                DEBUG(SSSDBG_OP_FAILURE, "C_GetSlotInfo failed\n");
-                return EIO;
-            }
-            DEBUG(SSSDBG_TRACE_ALL,
-                  "Description [%s] Manufacturer [%s] flags [%lu] "
-                  "removable [%s] token present [%s].\n",
-                  info.slotDescription, info.manufacturerID, info.flags,
-                  (info.flags & CKF_REMOVABLE_DEVICE) ? "true": "false",
-                  (info.flags & CKF_TOKEN_PRESENT) ? "true": "false");
-            if ((info.flags & CKF_REMOVABLE_DEVICE)
-                    && (info.flags & CKF_TOKEN_PRESENT)) {
-                break;
-            }
-        } while (true);
-    }
+        }
+
+        rv = module->C_GetSlotInfo(*slot_id, &info);
+        if (rv != CKR_OK) {
+            DEBUG(SSSDBG_OP_FAILURE, "C_GetSlotInfo failed\n");
+            return EIO;
+        }
+        DEBUG(SSSDBG_TRACE_ALL,
+              "Description [%s] Manufacturer [%s] flags [%lu] "
+              "removable [%s] token present [%s].\n",
+              info.slotDescription, info.manufacturerID, info.flags,
+              (info.flags & CKF_REMOVABLE_DEVICE) ? "true": "false",
+              (info.flags & CKF_TOKEN_PRESENT) ? "true": "false");
+
+        /* Check if really a token is present */
+        if ((info.flags & CKF_REMOVABLE_DEVICE)
+                && (info.flags & CKF_TOKEN_PRESENT)) {
+            break;
+        }
+    } while (true);
 
     return EOK;
 }
@@ -1584,7 +1587,8 @@ static errno_t wait_for_card(CK_FUNCTION_LIST *module, CK_SLOT_ID *slot_id)
 errno_t do_card(TALLOC_CTX *mem_ctx, struct p11_ctx *p11_ctx,
                 enum op_mode mode, const char *pin,
                 const char *module_name_in, const char *token_name_in,
-                const char *key_id_in, const char *uri_str, char **_multi)
+                const char *key_id_in, const char *label_in,
+                const char *uri_str, char **_multi)
 {
     int ret;
     size_t c;
@@ -1842,11 +1846,13 @@ errno_t do_card(TALLOC_CTX *mem_ctx, struct p11_ctx *p11_ctx,
     DLIST_FOR_EACH(item, all_cert_list) {
         /* Check if we found the certificates we needed for authentication or
          * the requested ones for pre-auth. For authentication all attributes
-         * must be given and match, for pre-auth only the given ones must
-         * match. */
-        DEBUG(SSSDBG_TRACE_ALL, "%s %s %s %s %s %s.\n",
+         * except the label must be given and match. The label is optional for
+         * authentication but if given it must match as well. For pre-auth
+         * only the given ones must match. */
+        DEBUG(SSSDBG_TRACE_ALL, "%s %s %s %s %s %s %s.\n",
               module_name_in, module_file_name, token_name_in, token_name,
-              key_id_in, item->id);
+              key_id_in, label_in == NULL ? "- no label given-" : label_in,
+              item->id);
 
         if ((mode == OP_AUTH
                 && module_name_in != NULL
@@ -1854,6 +1860,9 @@ errno_t do_card(TALLOC_CTX *mem_ctx, struct p11_ctx *p11_ctx,
                 && key_id_in != NULL
                 && item->id != NULL
                 && strcmp(key_id_in, item->id) == 0
+                && (label_in == NULL
+                    || (label_in != NULL && item->label != NULL
+                        && strcmp(label_in, item->label) == 0))
                 && strcmp(token_name_in, token_name) == 0
                 && strcmp(module_name_in, module_file_name) == 0)
             || (mode == OP_PREAUTH
