@@ -23,6 +23,7 @@
 */
 
 #include <sys/types.h>
+#include <ctype.h>
 #include <unistd.h>
 #include <sys/stat.h>
 #include <popt.h>
@@ -529,7 +530,8 @@ copy_smb_file_to_gpo_cache(SMBCCTX *smbc_ctx,
                            const char *smb_cse_suffix)
 {
     char *smb_uri = NULL;
-    SMBCFILE *file;
+    char *gpt_main_folder = NULL;
+    SMBCFILE *file = NULL;
     int ret;
     uint8_t *buf = NULL;
     int buflen = 0;
@@ -554,10 +556,41 @@ copy_smb_file_to_gpo_cache(SMBCCTX *smbc_ctx,
     errno = 0;
     file = smbc_getFunctionOpen(smbc_ctx)(smbc_ctx, smb_uri, O_RDONLY, 0755);
     if (file == NULL) {
-        ret = errno;
-        DEBUG(SSSDBG_CRIT_FAILURE, "smbc_getFunctionOpen failed [%d][%s]\n",
-              ret, strerror(ret));
-        goto done;
+        // ENOENT: A directory component in pathname does not exist
+        if (errno == ENOENT) {
+            /*
+             * DCs may use upper case names for the main folder, where GPTs are
+             * stored. libsmbclient does not allow us to request case insensitive
+             * file name lookups on DCs with case sensitive file systems.
+             */
+            gpt_main_folder = strstr(smb_uri, "/Machine/");
+            if (gpt_main_folder == NULL) {
+                /* At this moment we do not use any GPO from user settings,
+                 * but it can change in the future so let's keep the following
+                 * line around to make this part of the code 'just work' also
+                 * with the user GPO settings. */
+                gpt_main_folder = strstr(smb_uri, "/User/");
+            }
+            if (gpt_main_folder != NULL) {
+                ++gpt_main_folder;
+                while (gpt_main_folder != NULL && *gpt_main_folder != '/') {
+                    *gpt_main_folder = toupper(*gpt_main_folder);
+                    ++gpt_main_folder;
+                }
+
+                DEBUG(SSSDBG_TRACE_FUNC, "smb_uri: %s\n", smb_uri);
+
+                errno = 0;
+                file = smbc_getFunctionOpen(smbc_ctx)(smbc_ctx, smb_uri, O_RDONLY, 0755);
+            }
+        }
+
+        if (file == NULL) {
+            ret = errno;
+            DEBUG(SSSDBG_CRIT_FAILURE, "smbc_getFunctionOpen failed [%d][%s]\n",
+                  ret, strerror(ret));
+            goto done;
+        }
     }
 
     buf = talloc_array(tmp_ctx, uint8_t, SMB_BUFFER_SIZE);
@@ -587,6 +620,10 @@ copy_smb_file_to_gpo_cache(SMBCCTX *smbc_ctx,
     }
 
  done:
+    if (file != NULL) {
+        smbc_getFunctionClose(smbc_ctx)(smbc_ctx, file);
+    }
+
     talloc_free(tmp_ctx);
     return ret;
 }
@@ -687,6 +724,7 @@ main(int argc, const char *argv[])
     int opt;
     poptContext pc;
     int debug_fd = -1;
+    const char *opt_logger = NULL;
     errno_t ret;
     int sysvol_gpt_version;
     int result;
@@ -710,6 +748,7 @@ main(int argc, const char *argv[])
         {"debug-to-stderr", 0, POPT_ARG_NONE | POPT_ARGFLAG_DOC_HIDDEN,
          &debug_to_stderr, 0,
          _("Send the debug output to stderr directly."), NULL },
+        SSSD_LOGGER_OPTS
         POPT_TABLEEND
     };
 
@@ -731,7 +770,7 @@ main(int argc, const char *argv[])
 
     DEBUG_INIT(debug_level);
 
-    debug_prg_name = talloc_asprintf(NULL, "[sssd[gpo_child[%d]]]", getpid());
+    debug_prg_name = talloc_asprintf(NULL, "gpo_child[%d]", getpid());
     if (debug_prg_name == NULL) {
         DEBUG(SSSDBG_CRIT_FAILURE, "talloc_asprintf failed.\n");
         goto fail;
@@ -742,8 +781,11 @@ main(int argc, const char *argv[])
         if (ret != EOK) {
             DEBUG(SSSDBG_CRIT_FAILURE, "set_debug_file_from_fd failed.\n");
         }
+        opt_logger = sss_logger_str[FILES_LOGGER];
     }
 
+    sss_set_logger(opt_logger);
+
     DEBUG(SSSDBG_TRACE_FUNC, "gpo_child started.\n");
 
     main_ctx = talloc_new(NULL);
