@@ -214,12 +214,12 @@ static void be_ptask_schedule(struct be_ptask *task,
             return;
         }
 
-        delay = task->period;
-
-        if (backoff_allowed(task) && task->period * 2 <= task->max_backoff) {
+        if (backoff_allowed(task)) {
             /* double the period for the next execution */
-            task->period *= 2;
+            task->period = MIN(task->period * 2, task->max_backoff);
         }
+
+        delay = task->period;
         break;
     }
 
@@ -251,7 +251,7 @@ static void be_ptask_schedule(struct be_ptask *task,
     task->timer = tevent_add_timer(task->ev, task, tv, be_ptask_execute, task);
     if (task->timer == NULL) {
         /* nothing we can do about it */
-        DEBUG(SSSDBG_CRIT_FAILURE, "FATAL: Unable to schedule task [%s]\n",
+        DEBUG(SSSDBG_CRIT_FAILURE, "Unable to schedule task [%s]\n",
                                     task->name);
         be_ptask_disable(task);
     }
