@@ -88,6 +88,7 @@ dp_init_interface(struct data_provider *provider)
             SBUS_ASYNC(METHOD, sssd_dataprovider, pamHandler, dp_pam_handler_send, dp_pam_handler_recv, provider),
             SBUS_ASYNC(METHOD, sssd_dataprovider, sudoHandler, dp_sudo_handler_send, dp_sudo_handler_recv, provider),
             SBUS_ASYNC(METHOD, sssd_dataprovider, hostHandler, dp_host_handler_send, dp_host_handler_recv, provider),
+            SBUS_ASYNC(METHOD, sssd_dataprovider, resolverHandler, dp_resolver_handler_send, dp_resolver_handler_recv, provider),
             SBUS_ASYNC(METHOD, sssd_dataprovider, getDomains, dp_subdomains_handler_send, dp_subdomains_handler_recv, provider),
             SBUS_ASYNC(METHOD, sssd_dataprovider, getAccountInfo, dp_get_account_info_send, dp_get_account_info_recv, provider),
             SBUS_ASYNC(METHOD, sssd_dataprovider, getAccountDomain, dp_get_account_domain_send, dp_get_account_domain_recv, provider)
@@ -108,7 +109,7 @@ dp_init_interface(struct data_provider *provider)
 
     ret = sbus_connection_add_path_map(provider->sbus_conn, paths);
     if (ret != EOK) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "Unable to add paths [%d]: %s\n",
+        DEBUG(SSSDBG_FATAL_FAILURE, "Unable to add paths [%d]: %s\n",
               ret, sss_strerror(ret));
     }
 
@@ -133,7 +134,6 @@ static int dp_destructor(struct data_provider *provider)
 struct dp_init_state {
     struct be_ctx *be_ctx;
     struct data_provider *provider;
-    char *sbus_name;
 };
 
 static void dp_init_done(struct tevent_req *subreq);
@@ -143,7 +143,8 @@ dp_init_send(TALLOC_CTX *mem_ctx,
              struct tevent_context *ev,
              struct be_ctx *be_ctx,
              uid_t uid,
-             gid_t gid)
+             gid_t gid,
+             const char *sbus_name)
 {
     struct dp_init_state *state;
     struct tevent_req *subreq;
@@ -176,13 +177,6 @@ dp_init_send(TALLOC_CTX *mem_ctx,
     state->provider->gid = gid;
     state->provider->be_ctx = be_ctx;
 
-    state->sbus_name = sss_iface_domain_bus(state, be_ctx->domain);
-    if (state->sbus_name == NULL) {
-        DEBUG(SSSDBG_FATAL_FAILURE, "Could not get sbus backend name.\n");
-        ret = ENOMEM;
-        goto done;
-    }
-
     /* Initialize data provider bus. Data provider can receive client
      * registration and other D-Bus methods. However no data provider
      * request will be executed as long as the modules and targets
@@ -191,11 +185,11 @@ dp_init_send(TALLOC_CTX *mem_ctx,
     talloc_set_destructor(state->provider, dp_destructor);
 
     subreq = sbus_server_create_and_connect_send(state->provider, ev,
-                                                 state->sbus_name,
-                                                 NULL, sbus_address, true, 1000,
-                                                 uid, gid);
+        sbus_name, NULL, sbus_address, true, 1000, uid, gid,
+        (sbus_server_on_connection_cb)dp_client_init,
+        (sbus_server_on_connection_data)state->provider);
     if (subreq == NULL) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "Unable to create subrequest!\n");
+        DEBUG(SSSDBG_FATAL_FAILURE, "Unable to create subrequest!\n");
         ret = ENOMEM;
         goto done;
     }
@@ -234,9 +228,6 @@ static void dp_init_done(struct tevent_req *subreq)
         return;
     }
 
-    sbus_server_set_on_connection(state->provider->sbus_server,
-                                  dp_client_init, state->provider);
-
     /* be_ctx->provider must be accessible from modules and targets */
     state->be_ctx->provider = talloc_steal(state->be_ctx, state->provider);
 
@@ -272,16 +263,10 @@ done:
 }
 
 errno_t dp_init_recv(TALLOC_CTX *mem_ctx,
-                     struct tevent_req *req,
-                     const char **_sbus_name)
+                     struct tevent_req *req)
 {
-    struct dp_init_state *state;
-    state = tevent_req_data(req, struct dp_init_state);
-
     TEVENT_REQ_RETURN_ON_ERROR(req);
 
-    *_sbus_name = talloc_steal(mem_ctx, state->sbus_name);
-
     return EOK;
 }
 
