@@ -51,6 +51,10 @@
 #define ONLINE_CB_RETRY 3
 #define ONLINE_CB_RETRY_MAX_DELAY 4
 
+#define OFFLINE_TIMEOUT_RANDOM_OFFSET 30
+#define OFFLINE_TIMEOUT_DEFAULT 60
+#define OFFLINE_TIMEOUT_MAX_DEFAULT 3600
+
 /* sssd.service */
 static errno_t
 data_provider_res_init(TALLOC_CTX *mem_ctx,
@@ -98,21 +102,42 @@ static int get_offline_timeout(struct be_ctx *ctx)
     int offline_timeout;
 
     ret = confdb_get_int(ctx->cdb, ctx->conf_path,
-                         CONFDB_DOMAIN_OFFLINE_TIMEOUT, 60,
+                         CONFDB_DOMAIN_OFFLINE_TIMEOUT,
+                         OFFLINE_TIMEOUT_DEFAULT,
                          &offline_timeout);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE,
               "Failed to get offline_timeout from confdb. "
-              "Will use 60 seconds.\n");
-        offline_timeout = 60;
+              "Will use %d seconds.\n", OFFLINE_TIMEOUT_DEFAULT);
+        offline_timeout = OFFLINE_TIMEOUT_DEFAULT;
     }
 
     return offline_timeout;
 }
 
+static int get_offline_timeout_max(struct be_ctx *ctx)
+{
+    int offline_timeout_max;
+    errno_t ret;
+
+    ret = confdb_get_int(ctx->cdb, ctx->conf_path,
+                         CONFDB_DOMAIN_OFFLINE_TIMEOUT_MAX,
+                         OFFLINE_TIMEOUT_MAX_DEFAULT,
+                         &offline_timeout_max);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CONF_SETTINGS,
+              "Failed to get offline_timeout_max from confdb. "
+              "Will use %d seconds.\n", OFFLINE_TIMEOUT_MAX_DEFAULT);
+        offline_timeout_max = OFFLINE_TIMEOUT_MAX_DEFAULT;
+    }
+
+    return offline_timeout_max;
+}
+
 void be_mark_offline(struct be_ctx *ctx)
 {
     int offline_timeout;
+    int offline_timeout_max;
     errno_t ret;
 
     DEBUG(SSSDBG_TRACE_INTERNAL, "Going offline!\n");
@@ -126,11 +151,16 @@ void be_mark_offline(struct be_ctx *ctx)
         DEBUG(SSSDBG_TRACE_INTERNAL, "Initialize check_if_online_ptask.\n");
 
         offline_timeout = get_offline_timeout(ctx);
+        offline_timeout_max = get_offline_timeout_max(ctx);
 
-        ret = be_ptask_create_sync(ctx, ctx,
-                                   offline_timeout, offline_timeout,
-                                   offline_timeout, 30, offline_timeout,
-                                   3600 /* max_backoff */,
+        ret = be_ptask_create_sync(ctx,
+                                   ctx,
+                                   offline_timeout,
+                                   offline_timeout,
+                                   offline_timeout,
+                                   OFFLINE_TIMEOUT_RANDOM_OFFSET,
+                                   offline_timeout,
+                                   offline_timeout_max,
                                    try_to_go_online,
                                    ctx, "Check if online (periodic)",
                                    BE_PTASK_OFFLINE_EXECUTE,
@@ -377,7 +407,7 @@ static void check_if_online(struct be_ctx *be_ctx, int delay)
                                   check_if_online_delayed, be_ctx);
 
     if (time_event == NULL) {
-        DEBUG(SSSDBG_OP_FAILURE,
+        DEBUG(SSSDBG_CRIT_FAILURE,
               "Scheduling check_if_online_delayed failed.\n");
         goto failed;
     }
@@ -390,7 +420,6 @@ static void check_if_online(struct be_ctx *be_ctx, int delay)
 
 failed:
     be_ctx->check_online_ref_count--;
-    DEBUG(SSSDBG_CRIT_FAILURE, "Failed to run a check_online test.\n");
 
     if (be_ctx->check_online_ref_count == 0) {
         reset_fo(be_ctx);
@@ -445,36 +474,6 @@ be_register_monitor_iface(struct sbus_connection *conn, struct be_ctx *be_ctx)
     return sbus_connection_add_path_map(be_ctx->mon_conn, paths);
 }
 
-static void be_process_finalize(struct tevent_context *ev,
-                                struct tevent_signal *se,
-                                int signum,
-                                int count,
-                                void *siginfo,
-                                void *private_data)
-{
-    struct be_ctx *be_ctx;
-
-    be_ctx = talloc_get_type(private_data, struct be_ctx);
-    talloc_free(be_ctx);
-    orderly_shutdown(0);
-}
-
-static errno_t be_process_install_sigterm_handler(struct be_ctx *be_ctx)
-{
-    struct tevent_signal *sige;
-
-    BlockSignals(false, SIGTERM);
-
-    sige = tevent_add_signal(be_ctx->ev, be_ctx, SIGTERM, SA_SIGINFO,
-                             be_process_finalize, be_ctx);
-    if (sige == NULL) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "tevent_add_signal failed.\n");
-        return ENOMEM;
-    }
-
-    return EOK;
-}
-
 static void dp_initialized(struct tevent_req *req);
 
 errno_t be_process_init(TALLOC_CTX *mem_ctx,
@@ -566,14 +565,15 @@ errno_t be_process_init(TALLOC_CTX *mem_ctx,
         goto done;
     }
 
-    /* Install signal handler */
-    ret = be_process_install_sigterm_handler(be_ctx);
-    if (ret != EOK) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "be_install_sigterm_handler failed.\n");
+    be_ctx->sbus_name = sss_iface_domain_bus(be_ctx, be_ctx->domain);
+    if (be_ctx->sbus_name == NULL) {
+        DEBUG(SSSDBG_FATAL_FAILURE, "Could not get sbus backend name.\n");
+        ret = ENOMEM;
         goto done;
     }
 
-    req = dp_init_send(be_ctx, be_ctx->ev, be_ctx, be_ctx->uid, be_ctx->gid);
+    req = dp_init_send(be_ctx, be_ctx->ev, be_ctx, be_ctx->uid, be_ctx->gid,
+                       be_ctx->sbus_name);
     if (req == NULL) {
         ret = ENOMEM;
         goto done;
@@ -620,7 +620,7 @@ static void dp_initialized(struct tevent_req *req)
 
     be_ctx = tevent_req_callback_data(req, struct be_ctx);
 
-    ret = dp_init_recv(be_ctx, req, &be_ctx->sbus_name);
+    ret = dp_init_recv(be_ctx, req);
     talloc_zfree(req);
     if (ret !=  EOK) {
         goto done;
@@ -636,7 +636,7 @@ static void dp_initialized(struct tevent_req *req)
 
     ret = be_register_monitor_iface(be_ctx->mon_conn, be_ctx);
     if (ret != EOK) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "Unable to register monitor interface "
+        DEBUG(SSSDBG_FATAL_FAILURE, "Unable to register monitor interface "
               "[%d]: %s\n", ret, sss_strerror(ret));
         goto done;
     }
@@ -731,6 +731,10 @@ int main(int argc, const char *argv[])
             poptPrintUsage(pc, stderr, 0);
             return 1;
     }
+    if (!is_valid_domain_name(be_domain)) {
+        fprintf(stderr, "\nInvalid --domain option.\n\n");
+        return 1;
+    }
 
     poptFreeContext(pc);
 
@@ -742,7 +746,7 @@ int main(int argc, const char *argv[])
 
     sss_set_logger(opt_logger);
 
-    srv_name = talloc_asprintf(NULL, "sssd[be[%s]]", be_domain);
+    srv_name = talloc_asprintf(NULL, "be[%s]", be_domain);
     if (!srv_name) return 2;
 
     confdb_path = talloc_asprintf(NULL, CONFDB_DOMAIN_PATH_TMPL, be_domain);
