@@ -63,7 +63,7 @@ int be_init_failover(struct be_ctx *ctx)
     }
 
     ctx->be_fo = talloc_zero(ctx, struct be_failover_ctx);
-    if (!ctx->be_fo) {
+    if (ctx->be_fo == NULL) {
         return ENOMEM;
     }
 
@@ -651,7 +651,7 @@ errno_t be_resolve_server_process(struct tevent_req *subreq,
         srvaddr = fo_get_server_hostent(state->srv);
         if (!srvaddr) {
             DEBUG(SSSDBG_CRIT_FAILURE,
-                  "FATAL: No hostent available for server (%s)\n",
+                  "No hostent available for server (%s)\n",
                   fo_get_server_str_name(state->srv));
             return EFAULT;
         }
@@ -884,7 +884,7 @@ errno_t be_res_init(struct be_ctx *ctx)
     }
 
     ctx->be_res = talloc_zero(ctx, struct be_resolv_ctx);
-    if (!ctx->be_res) {
+    if (ctx->be_res == NULL) {
         return ENOMEM;
     }
 
