@@ -300,9 +300,11 @@ static errno_t activate_child_timeout_handler(struct tevent_req *req,
 }
 
 errno_t set_extra_args(TALLOC_CTX *mem_ctx, struct krb5_ctx *krb5_ctx,
+                       struct sss_domain_info *domain,
                        const char ***krb5_child_extra_args)
 {
     const char **extra_args;
+    const char *krb5_realm;
     size_t c = 0;
     int ret;
 
@@ -336,9 +338,15 @@ errno_t set_extra_args(TALLOC_CTX *mem_ctx, struct krb5_ctx *krb5_ctx,
     }
     c++;
 
+    krb5_realm = krb5_ctx->realm;
+    if (domain != NULL && IS_SUBDOMAIN(domain) && dp_opt_get_bool(krb5_ctx->opts, KRB5_USE_SUBDOMAIN_REALM)) {
+        DEBUG(SSSDBG_CONF_SETTINGS, "Use subdomain realm %s.\n", domain->realm);
+        krb5_realm = domain->realm;
+    }
+
     if (krb5_ctx->realm != NULL) {
         extra_args[c] = talloc_asprintf(extra_args, "--"CHILD_OPT_REALM"=%s",
-                                        krb5_ctx->realm);
+                                        krb5_realm);
         if (extra_args[c] == NULL) {
             DEBUG(SSSDBG_OP_FAILURE, "talloc_asprintf failed.\n");
             ret = ENOMEM;
@@ -439,7 +447,7 @@ static errno_t fork_child(struct tevent_req *req)
     struct handle_child_state *state = tevent_req_data(req,
                                                      struct handle_child_state);
 
-    ret = set_extra_args(state, state->kr->krb5_ctx, &krb5_child_extra_args);
+    ret = set_extra_args(state, state->kr->krb5_ctx, state->kr->dom, &krb5_child_extra_args);
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE, "set_extra_args failed.\n");
         goto fail;
@@ -449,14 +457,14 @@ static errno_t fork_child(struct tevent_req *req)
     if (ret == -1) {
         ret = errno;
         DEBUG(SSSDBG_CRIT_FAILURE,
-              "pipe failed [%d][%s].\n", errno, strerror(errno));
+              "pipe (from) failed [%d][%s].\n", errno, strerror(errno));
         goto fail;
     }
     ret = pipe(pipefd_to_child);
     if (ret == -1) {
         ret = errno;
         DEBUG(SSSDBG_CRIT_FAILURE,
-              "pipe failed [%d][%s].\n", errno, strerror(errno));
+              "pipe (to) failed [%d][%s].\n", errno, strerror(errno));
         goto fail;
     }
 
@@ -633,6 +641,33 @@ int handle_child_recv(struct tevent_req *req, TALLOC_CTX *mem_ctx,
     return EOK;
 }
 
+static const char *krb5_child_response_type_to_str(int32_t type)
+{
+    switch (type) {
+    case SSS_PAM_ENV_ITEM:
+        return "Env variable to be set with pam_putenv(3)";
+    case SSS_PAM_USER_INFO:
+        return "Message to be displayed to the user";
+    case SSS_OTP:
+        return "Authtok was a OTP";
+    case SSS_PAM_TEXT_MSG:
+        return "Plain text message to be displayed to the user";
+    case SSS_PAM_OTP_INFO:
+        return "OTP info";
+    case SSS_PASSWORD_PROMPTING:
+        return "Password prompting is possible";
+    case SSS_CERT_AUTH_PROMPTING:
+        return "Certificate based authentication is available";
+    case SSS_KRB5_INFO_TGT_LIFETIME:
+        return "TGT lifetime info";
+    case SSS_KRB5_INFO_UPN:
+        return "UPN info";
+    }
+
+    DEBUG(SSSDBG_MINOR_FAILURE, "Unexpected response type %d\n", type);
+    return "-unexpected-";
+}
+
 errno_t
 parse_krb5_child_response(TALLOC_CTX *mem_ctx, uint8_t *buf, ssize_t len,
                           struct pam_data *pd, int pwd_exp_warning,
@@ -685,8 +720,11 @@ parse_krb5_child_response(TALLOC_CTX *mem_ctx, uint8_t *buf, ssize_t len,
         SAFEALIGN_COPY_INT32(&msg_type, buf+p, &p);
         SAFEALIGN_COPY_INT32(&msg_len, buf+p, &p);
 
-        DEBUG(SSSDBG_TRACE_LIBS, "child response [%d][%d][%d].\n",
-              msg_status, msg_type, msg_len);
+        DEBUG(SSSDBG_TRACE_LIBS, "child response: "
+              "status code: %d (%s), msg type: %d (%s), len: %d\n",
+              msg_status, sss_strerror(msg_status),
+              msg_type, krb5_child_response_type_to_str(msg_type),
+              msg_len);
 
         if (msg_len > len - p) {
             DEBUG(SSSDBG_CRIT_FAILURE, "message format error [%d] > [%zu].\n",
