@@ -63,6 +63,7 @@ enum krb5_opts {
     KRB5_USE_KDCINFO,
     KRB5_KDCINFO_LOOKAHEAD,
     KRB5_MAP_USER,
+    KRB5_USE_SUBDOMAIN_REALM,
 
     KRB5_OPTS
 };
@@ -124,7 +125,6 @@ struct krb5_ctx {
     struct dp_option *opts;
     struct krb5_service *service;
     struct krb5_service *kpasswd_service;
-    int child_debug_fd;
 
     sss_regexp_t *illegal_path_re;
 
@@ -196,13 +196,6 @@ int krb5_service_init(TALLOC_CTX *memctx, struct be_ctx *ctx,
 
 void remove_krb5_info_files_callback(void *pvt);
 
-void krb5_finalize(struct tevent_context *ev,
-                   struct tevent_signal *se,
-                   int signum,
-                   int count,
-                   void *siginfo,
-                   void *private_data);
-
 errno_t remove_krb5_info_files(TALLOC_CTX *mem_ctx, const char *realm);
 
 errno_t krb5_get_simple_upn(TALLOC_CTX *mem_ctx, struct krb5_ctx *krb5_ctx,
@@ -245,5 +238,6 @@ krb5_error_code copy_keytab_into_memory(TALLOC_CTX *mem_ctx, krb5_context kctx,
                                         krb5_keytab *_mem_keytab);
 
 errno_t set_extra_args(TALLOC_CTX *mem_ctx, struct krb5_ctx *krb5_ctx,
+                       struct sss_domain_info *domain,
                        const char ***krb5_child_extra_args);
 #endif /* __KRB5_COMMON_H__ */
