@@ -287,7 +287,7 @@ char *expand_ccname_template(TALLOC_CTX *mem_ctx, struct krb5child_req *kr,
                 name = sss_output_name(tmp_ctx, kr->pd->user, case_sensitive, 0);
                 if (name == NULL) {
                     DEBUG(SSSDBG_CRIT_FAILURE,
-                          "sss_get_cased_name failed\n");
+                          "sss_output_name failed\n");
                     goto done;
                 }
 
