@@ -175,6 +175,8 @@ enum sdap_basic_opt {
     SDAP_SUDO_INCLUDE_REGEXP,
     SDAP_AUTOFS_SEARCH_BASE,
     SDAP_AUTOFS_MAP_MASTER_NAME,
+    SDAP_IPHOST_SEARCH_BASE,
+    SDAP_IPNETWORK_SEARCH_BASE,
     SDAP_SCHEMA,
     SDAP_PWMODIFY_MODE,
     SDAP_OFFLINE_TIMEOUT,
@@ -192,6 +194,7 @@ enum sdap_basic_opt {
     SDAP_SASL_AUTHID,
     SDAP_SASL_REALM,
     SDAP_SASL_MINSSF,
+    SDAP_SASL_MAXSSF,
     SDAP_KRB5_KEYTAB,
     SDAP_KRB5_KINIT,
     SDAP_KRB5_KDC,
@@ -221,6 +224,7 @@ enum sdap_basic_opt {
     SDAP_DEREF_THRESHOLD,
     SDAP_SASL_CANONICALIZE,
     SDAP_EXPIRE_TIMEOUT,
+    SDAP_EXPIRE_OFFSET,
     SDAP_DISABLE_PAGING,
     SDAP_IDMAP_LOWER,
     SDAP_IDMAP_UPPER,
@@ -236,6 +240,7 @@ enum sdap_basic_opt {
     SDAP_MAX_ID,
     SDAP_PWDLOCKOUT_DN,
     SDAP_WILDCARD_LIMIT,
+    SDAP_LIBRARY_DEBUG_LEVEL,
 
     SDAP_OPTS_BASIC /* opts counter */
 };
@@ -364,6 +369,24 @@ enum sdap_service_attrs {
     SDAP_OPTS_SERVICES /* attrs counter */
 };
 
+enum sdap_iphost_entry_attrs {
+    SDAP_OC_IPHOST = 0,
+    SDAP_AT_IPHOST_NAME,
+    SDAP_AT_IPHOST_NUMBER,
+    SDAP_AT_IPHOST_USN,
+
+    SDAP_OPTS_IPHOST /* attrs counter */
+};
+
+enum sdap_ipnetwork_entry_attrs {
+    SDAP_OC_IPNETWORK = 0,
+    SDAP_AT_IPNETWORK_NAME,
+    SDAP_AT_IPNETWORK_NUMBER,
+    SDAP_AT_IPNETWORK_USN,
+
+    SDAP_OPTS_IPNETWORK /* attrs counter */
+};
+
 enum sdap_autofs_map_attrs {
     SDAP_OC_AUTOFS_MAP,
     SDAP_AT_AUTOFS_MAP_NAME,
@@ -425,6 +448,8 @@ struct sdap_domain {
     struct sdap_search_base **host_search_bases;
     struct sdap_search_base **sudo_search_bases;
     struct sdap_search_base **service_search_bases;
+    struct sdap_search_base **iphost_search_bases;
+    struct sdap_search_base **ipnetwork_search_bases;
     struct sdap_search_base **autofs_search_bases;
 
     struct sdap_domain *next, *prev;
@@ -466,6 +491,8 @@ struct sdap_options {
     struct sdap_attr_map *netgroup_map;
     struct sdap_attr_map *host_map;
     struct sdap_attr_map *service_map;
+    struct sdap_attr_map *iphost_map;
+    struct sdap_attr_map *ipnetwork_map;
 
     /* ID-mapping support */
     struct sdap_idmap_ctx *idmap_ctx;
@@ -499,6 +526,7 @@ struct sdap_options {
     bool support_matching_rule;
     enum dc_functional_level dc_functional_level;
     const char *schema_basedn;
+    bool allow_remote_domain_local_groups;
 
     /* Certificate mapping support */
     struct sdap_certmap_ctx *sdap_certmap_ctx;
@@ -512,6 +540,8 @@ struct sdap_server_opts {
     char *max_group_value;
     char *max_service_value;
     char *max_sudo_value;
+    char *max_iphost_value;
+    char *max_ipnetwork_value;
 };
 
 struct sdap_id_ctx;
@@ -594,6 +624,8 @@ errno_t sdap_parse_deref(TALLOC_CTX *mem_ctx,
                          LDAPDerefRes *dref,
                          struct sdap_deref_attrs ***_deref_res);
 
+void setup_ldap_debug(struct dp_option *basic_opts);
+
 errno_t setup_tls_config(struct dp_option *basic_opts);
 
 int sdap_set_rootdse_supported_lists(struct sysdb_attrs *rootdse,
