@@ -27,14 +27,14 @@
 /* shared non-async user functions */
 
 errno_t sdap_fallback_local_user(TALLOC_CTX *memctx,
-                             struct sdap_options *opts,
-                             const char *name, uid_t uid,
-                             struct sysdb_attrs ***reply);
+                                 const char *name, uid_t uid,
+                                 struct sysdb_attrs ***reply);
 
 int sdap_save_user(TALLOC_CTX *memctx,
                    struct sdap_options *opts,
                    struct sss_domain_info *dom,
                    struct sysdb_attrs *attrs,
+                   struct sysdb_attrs *mapped_attrs,
                    char **_usn_value,
                    time_t now);
 
