@@ -26,7 +26,6 @@
 #include "util/util.h"
 #include "responder/common/responder.h"
 #include "responder/common/cache_req/cache_req_private.h"
-#include "responder/common/cache_req/cache_req_private.h"
 #include "responder/common/cache_req/cache_req_plugin.h"
 
 static const struct cache_req_plugin *
@@ -53,17 +52,24 @@ cache_req_get_plugin(enum cache_req_type type)
         &cache_req_enum_users,
         &cache_req_enum_groups,
         &cache_req_enum_svc,
+        &cache_req_enum_ip_hosts,
+        &cache_req_enum_ip_networks,
 
         &cache_req_svc_by_name,
         &cache_req_svc_by_port,
 
         &cache_req_netgroup_by_name,
 
-        &cache_req_host_by_name,
+        &cache_req_ssh_host_id_by_name,
 
         &cache_req_autofs_map_entries,
         &cache_req_autofs_map_by_name,
-        &cache_req_autofs_entry_by_name
+        &cache_req_autofs_entry_by_name,
+
+        &cache_req_ip_host_by_name,
+        &cache_req_ip_host_by_addr,
+        &cache_req_ip_network_by_name,
+        &cache_req_ip_network_by_addr,
     };
 
     if (type >= CACHE_REQ_SENTINEL) {
@@ -121,6 +127,7 @@ cache_req_create(TALLOC_CTX *mem_ctx,
                  enum cache_req_dom_type req_dom_type)
 {
     struct cache_req *cr;
+    bool bypass_cache;
     errno_t ret;
 
     cr = talloc_zero(mem_ctx, struct cache_req);
@@ -144,15 +151,22 @@ cache_req_create(TALLOC_CTX *mem_ctx,
         return NULL;
     }
 
-    cr->cache_first = rctx->cache_first;
-    cr->bypass_cache = cr->plugin->bypass_cache || cr->data->bypass_cache;
-    cr->bypass_dp = cr->data->bypass_dp;
-    if (cr->bypass_cache && cr->bypass_dp) {
+    bypass_cache = cr->plugin->bypass_cache || cr->data->bypass_cache;
+    if (bypass_cache && cr->data->bypass_dp) {
         CACHE_REQ_DEBUG(SSSDBG_CRIT_FAILURE, cr,
                         "Cannot bypass cache and dp at the same time!");
         talloc_free(cr);
         return NULL;
     }
+    if (rctx->cache_first) {
+        cr->cache_behavior = CACHE_REQ_CACHE_FIRST;
+    }
+    /* it is ok to override cache_first here */
+    if (bypass_cache) {
+        cr->cache_behavior = CACHE_REQ_BYPASS_CACHE;
+    } else if (cr->data->bypass_dp) {
+        cr->cache_behavior = CACHE_REQ_BYPASS_PROVIDER;
+    }
 
     return cr;
 }
@@ -493,8 +507,8 @@ static struct tevent_req *cache_req_locate_dom_send(TALLOC_CTX *mem_ctx,
     subreq = cache_req_search_send(state,
                                    state->ev,
                                    state->cr,
-                                   false,       /* Don't bypass cache */
-                                   true);       /* Do bypass DP */
+                                   false,
+                                   true);
     if (subreq == NULL) {
         ret = ENOMEM;
         goto immediately;
@@ -677,8 +691,8 @@ struct cache_req_search_domains_state {
     size_t num_results;
     bool check_next;
     bool dp_success;
-    bool bypass_cache;
-    bool bypass_dp;
+    bool first_iteration;
+    enum cache_req_behavior cache_behavior;
 };
 
 static errno_t cache_req_search_domains_next(struct tevent_req *req);
@@ -695,9 +709,7 @@ cache_req_search_domains_send(TALLOC_CTX *mem_ctx,
                               struct cache_req *cr,
                               struct cache_req_domain *cr_domain,
                               bool check_next,
-                              bool first_iteration,
-                              bool bypass_cache,
-                              bool bypass_dp)
+                              bool first_iteration)
 {
     struct tevent_req *req;
     struct cache_req_search_domains_state *state = NULL;
@@ -717,12 +729,12 @@ cache_req_search_domains_send(TALLOC_CTX *mem_ctx,
     state->req_domains = cr_domain;
     state->check_next = check_next;
     state->dp_success = true;
-    state->bypass_cache = bypass_cache;
-    state->bypass_dp = bypass_dp;
+    state->first_iteration = first_iteration;
 
     if (cr->plugin->dp_get_domain_send_fn != NULL
             && ((state->check_next && cr_domain->next != NULL)
-                || (state->bypass_cache && !first_iteration))) {
+                || ((state->cr->cache_behavior == CACHE_REQ_CACHE_FIRST)
+                    && !first_iteration))) {
         /* If the request is not qualified with a domain name AND
          * there are multiple domains to search OR if this is the second
          * pass during the "check-cache-first" schema, it makes sense
@@ -814,7 +826,8 @@ static errno_t cache_req_search_domains_next(struct tevent_req *req)
         }
 
         subreq = cache_req_search_send(state, state->ev, cr,
-                                       state->bypass_cache, state->bypass_dp);
+                                       state->first_iteration,
+                                       false);
         if (subreq == NULL) {
             return ENOMEM;
         }
@@ -961,6 +974,13 @@ static void cache_req_search_domains_done(struct tevent_req *subreq)
     case ERR_ID_OUTSIDE_RANGE:
     case ENOENT:
         if (state->check_next == false) {
+            if (state->cr->data->propogate_offline_status && !state->dp_success) {
+                /* Not found and data provider request failed so we were
+                 * unable to fetch the data. */
+                ret = ERR_OFFLINE;
+                goto done;
+            }
+
             /* Not found. */
             ret = ENOENT;
             goto done;
@@ -989,6 +1009,12 @@ done:
     case EAGAIN:
         break;
     default:
+        if (ret == ENOENT && state->cr->data->propogate_offline_status
+                && !state->dp_success) {
+            /* Not found and data provider request failed so we were
+             * unable to fetch the data. */
+            ret = ERR_OFFLINE;
+        }
         tevent_req_error(req, ret);
         break;
     }
@@ -1018,66 +1044,6 @@ cache_req_search_domains_recv(TALLOC_CTX *mem_ctx,
     return EOK;
 }
 
-/**
- * Return true if we should issue another search.
- */
-static bool cache_req_search_schema(struct cache_req *cr,
-                                    const char *input_domain,
-                                    bool first_iteration,
-                                    bool *_bypass_cache,
-                                    bool *_bypass_dp)
-{
-    bool bypass_cache;
-    bool bypass_dp;
-
-    if (cr->bypass_cache) {
-        /* The caller wants to contact Data Provider first
-         * or it is inferred by cache_req plug-in. */
-        bypass_cache = true;
-        bypass_dp = false;
-
-        if (!first_iteration) {
-            return false;
-        }
-    } else if (cr->bypass_dp) {
-        /* The caller wants to lookup only in the cache */
-        bypass_cache = false;
-        bypass_dp = true;
-
-        if (!first_iteration) {
-            return false;
-        }
-    } else if (input_domain != NULL) {
-        /* We will search only one domain. */
-        bypass_cache = false;
-        bypass_dp = false;
-
-        if (!first_iteration) {
-            return false;
-        }
-     } else if (!cr->cache_first) {
-        /* We will search cache and on cache-miss
-         * contact domain provider sequentially. */
-        bypass_cache = false;
-        bypass_dp = false;
-
-        if (!first_iteration) {
-            return false;
-        }
-    } else {
-        /* We will first search the cache in all domains. If we don't get
-         * any match we will then contact Data Provider starting with the
-         * first domain again. */
-        bypass_cache = first_iteration ? false : true;
-        bypass_dp = first_iteration ? true : false;
-    }
-
-    *_bypass_cache = bypass_cache;
-    *_bypass_dp = bypass_dp;
-
-    return true;
-}
-
 struct cache_req_state {
     /* input data */
     struct tevent_context *ev;
@@ -1106,14 +1072,13 @@ static void cache_req_domains_updated(struct tevent_req *subreq);
 static void cache_req_input_parsed(struct tevent_req *subreq);
 
 static errno_t cache_req_select_domains(struct tevent_req *req,
-                                        const char *domain_name);
+                                        const char *domain_name,
+                                        char **requested_domains);
 
 static errno_t
 cache_req_search_domains(struct tevent_req *req,
                          struct cache_req_domain *oredered_domain,
-                         bool check_next,
-                         bool bypass_cache,
-                         bool bypass_dp);
+                         bool check_next);
 
 static void cache_req_process_result(struct tevent_req *subreq);
 
@@ -1166,7 +1131,8 @@ struct tevent_req *cache_req_send(TALLOC_CTX *mem_ctx,
         goto done;
     }
 
-    ret = cache_req_select_domains(req, state->domain_name);
+    ret = cache_req_select_domains(req, state->domain_name,
+                                   cr->data->requested_domains);
 
 done:
     if (ret == EOK) {
@@ -1221,7 +1187,7 @@ static errno_t cache_req_process_input(TALLOC_CTX *mem_ctx,
     subreq = sss_parse_inp_send(mem_ctx, cr->rctx, default_domain,
                                 cr->data->name.input);
     if (subreq == NULL) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "Unable to create tevent request!\n");
+        DEBUG(SSSDBG_CRIT_FAILURE, "sss_parse_inp_send() failed\n");
         return ENOMEM;
     }
 
@@ -1279,7 +1245,8 @@ static void cache_req_domains_updated(struct tevent_req *subreq)
     }
 
 immediately:
-    ret = cache_req_select_domains(req, state->domain_name);
+    ret = cache_req_select_domains(req, state->domain_name,
+                                   state->cr->data->requested_domains);
 
 done:
     if (ret != EOK && ret != EAGAIN) {
@@ -1324,7 +1291,8 @@ static void cache_req_input_parsed(struct tevent_req *subreq)
     }
 
     state->domain_name = domain;
-    ret = cache_req_select_domains(req, domain);
+    ret = cache_req_select_domains(req, domain,
+                                   state->cr->data->requested_domains);
     if (ret != EAGAIN) {
         tevent_req_error(req, ret);
         return;
@@ -1332,28 +1300,28 @@ static void cache_req_input_parsed(struct tevent_req *subreq)
 }
 
 static errno_t cache_req_select_domains(struct tevent_req *req,
-                                        const char *domain_name)
+                                        const char *domain_name,
+                                        char **requested_domains)
 {
     struct cache_req_state *state = NULL;
     struct cache_req_domain *cr_domain;
     bool check_next;
-    bool bypass_cache;
-    bool bypass_dp;
-    bool search;
     errno_t ret;
 
     state = tevent_req_data(req, struct cache_req_state);
 
-    search = cache_req_search_schema(state->cr, domain_name,
-                                     state->first_iteration,
-                                     &bypass_cache, &bypass_dp);
-    if (!search) {
-        /* We're done here. */
-        return EOK;
+    if ((state->cr->cache_behavior != CACHE_REQ_CACHE_FIRST)
+        || (domain_name != NULL)) {
+
+        if (!state->first_iteration) {
+            /* We're done here. */
+            return EOK;
+        }
     }
 
     ret = cache_req_domain_copy_cr_domains(state,
                                            state->cr->rctx->cr_domains,
+                                           requested_domains,
                                            &state->cr_domains);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE, "cache_req_copy_cr_domains() failed\n");
@@ -1378,31 +1346,46 @@ static errno_t cache_req_select_domains(struct tevent_req *req,
         check_next = true;
     }
 
-    return cache_req_search_domains(req, cr_domain, check_next,
-                                    bypass_cache, bypass_dp);
+    return cache_req_search_domains(req, cr_domain, check_next);
 }
 
 static errno_t
 cache_req_search_domains(struct tevent_req *req,
                          struct cache_req_domain *cr_domain,
-                         bool check_next,
-                         bool bypass_cache,
-                         bool bypass_dp)
+                         bool check_next)
 {
     struct tevent_req *subreq;
     struct cache_req_state *state = NULL;
+    const char *cache_action;
+    const char *provider_action;
 
     state = tevent_req_data(req, struct cache_req_state);
 
+    switch (state->cr->cache_behavior) {
+    case CACHE_REQ_CACHE_FIRST:
+        cache_action = (state->first_iteration) ? "check" : "bypass";
+        provider_action = (state->first_iteration) ? "bypass" : "check";
+        break;
+    case CACHE_REQ_BYPASS_CACHE:
+        cache_action = "bypass";
+        provider_action = "check";
+        break;
+    case CACHE_REQ_BYPASS_PROVIDER:
+        cache_action = "check";
+        provider_action = "bypass";
+        break;
+    default:
+        cache_action = "check";
+        provider_action = "check";
+        break;
+    }
     CACHE_REQ_DEBUG(SSSDBG_TRACE_FUNC, state->cr,
                     "Search will %s the cache and %s the data provider\n",
-                    bypass_cache ? "bypass" : "check",
-                    bypass_dp ? "bypass" : "check");
+                    cache_action, provider_action);
 
     subreq = cache_req_search_domains_send(state, state->ev, state->cr,
                                            cr_domain, check_next,
-                                           state->first_iteration,
-                                           bypass_cache, bypass_dp);
+                                           state->first_iteration);
     if (subreq == NULL) {
         return ENOMEM;
     }
@@ -1427,7 +1410,8 @@ static void cache_req_process_result(struct tevent_req *subreq)
     if (ret == ENOENT && state->first_iteration) {
         /* Try again different search schema. */
         state->first_iteration = false;
-        ret = cache_req_select_domains(req, state->domain_name);
+        ret = cache_req_select_domains(req, state->domain_name,
+                                       state->cr->data->requested_domains);
         if (ret == EOK) {
             /* We're done searching and we have found nothing. */
             ret = ENOENT;
@@ -1440,7 +1424,8 @@ static void cache_req_process_result(struct tevent_req *subreq)
                 if (cache_req_assume_upn(state->cr)) {
                     /* Try UPN now. */
                     state->first_iteration = true;
-                    ret = cache_req_select_domains(req, NULL);
+                    ret = cache_req_select_domains(req, NULL,
+                                            state->cr->data->requested_domains);
                 }
             }
         }
