@@ -167,6 +167,13 @@ void
 cache_req_data_set_bypass_dp(struct cache_req_data *data,
                              bool bypass_dp);
 
+void
+cache_req_data_set_requested_domains(struct cache_req_data *data,
+                                     char **requested_domains);
+
+void
+cache_req_data_set_propogate_offline_status(struct cache_req_data *data,
+                                            bool propogate_offline_status);
 
 enum cache_req_type
 cache_req_data_get_type(struct cache_req_data *data);
@@ -270,6 +277,19 @@ cache_req_user_by_name_attrs_send(TALLOC_CTX *mem_ctx,
 #define cache_req_user_by_name_attrs_recv(mem_ctx, req, _result) \
     cache_req_single_domain_recv(mem_ctx, req, _result)
 
+struct tevent_req *
+cache_req_user_by_upn_send(TALLOC_CTX *mem_ctx,
+                           struct tevent_context *ev,
+                           struct resp_ctx *rctx,
+                           struct sss_nc_ctx *ncache,
+                           int cache_refresh_percent,
+                           enum cache_req_dom_type req_dom_type,
+                           const char *domain,
+                           const char *upn);
+
+#define cache_req_user_by_upn_recv(mem_ctx, req, _result) \
+    cache_req_single_domain_recv(mem_ctx, req, _result);
+
 struct tevent_req *
 cache_req_user_by_id_send(TALLOC_CTX *mem_ctx,
                           struct tevent_context *ev,
