@@ -311,12 +311,18 @@ extern const struct cache_req_plugin cache_req_object_by_id;
 extern const struct cache_req_plugin cache_req_enum_users;
 extern const struct cache_req_plugin cache_req_enum_groups;
 extern const struct cache_req_plugin cache_req_enum_svc;
+extern const struct cache_req_plugin cache_req_enum_ip_hosts;
+extern const struct cache_req_plugin cache_req_enum_ip_networks;
 extern const struct cache_req_plugin cache_req_svc_by_name;
 extern const struct cache_req_plugin cache_req_svc_by_port;
 extern const struct cache_req_plugin cache_req_netgroup_by_name;
-extern const struct cache_req_plugin cache_req_host_by_name;
+extern const struct cache_req_plugin cache_req_ssh_host_id_by_name;
 extern const struct cache_req_plugin cache_req_autofs_map_entries;
 extern const struct cache_req_plugin cache_req_autofs_map_by_name;
 extern const struct cache_req_plugin cache_req_autofs_entry_by_name;
+extern const struct cache_req_plugin cache_req_ip_host_by_name;
+extern const struct cache_req_plugin cache_req_ip_host_by_addr;
+extern const struct cache_req_plugin cache_req_ip_network_by_name;
+extern const struct cache_req_plugin cache_req_ip_network_by_addr;
 
 #endif /* _CACHE_REQ_PLUGIN_H_ */
