@@ -301,12 +301,15 @@ struct tevent_req *
 cache_req_search_send(TALLOC_CTX *mem_ctx,
                       struct tevent_context *ev,
                       struct cache_req *cr,
-                      bool bypass_cache,
-                      bool bypass_dp)
+                      bool first_iteration,
+                      bool cache_only_override)
 {
     struct cache_req_search_state *state;
     enum cache_object_status status;
     struct tevent_req *req;
+    bool bypass_cache = false;
+    bool bypass_dp = false;
+    bool skip_refresh = false;
     errno_t ret;
 
     req = tevent_req_create(mem_ctx, &state, struct cache_req_search_state);
@@ -325,6 +328,26 @@ cache_req_search_send(TALLOC_CTX *mem_ctx,
         goto done;
     }
 
+    if (cache_only_override) {
+        bypass_dp = true;
+    } else {
+        switch (cr->cache_behavior) {
+        case CACHE_REQ_CACHE_FIRST:
+            bypass_cache = first_iteration ? false : true;
+            bypass_dp = first_iteration ? true : false;
+            break;
+        case CACHE_REQ_BYPASS_CACHE:
+            bypass_cache = true;
+            break;
+        case CACHE_REQ_BYPASS_PROVIDER:
+            bypass_dp = true;
+            skip_refresh = true;
+            break;
+        default:
+            break;
+        }
+    }
+
     /* If bypass_cache is enabled we always contact data provider before
      * searching the cache. Thus we set expiration status to missing,
      * which will trigger data provider request later.
@@ -349,11 +372,12 @@ cache_req_search_send(TALLOC_CTX *mem_ctx,
             goto done;
         }
 
-        /* If bypass_dp is true but we found the object in this domain,
-         * we will contact the data provider anyway to refresh it so
-         * we can return it without searching the rest of the domains.
+        /* For the CACHE_REQ_CACHE_FIRST case, if bypass_dp is true but we
+         * found the object in this domain, we will contact the data provider
+         * anyway to refresh it so we can return it without searching the rest
+         * of the domains.
          */
-        if (status != CACHE_OBJECT_MISSING) {
+        if (status != CACHE_OBJECT_MISSING && !skip_refresh) {
             CACHE_REQ_DEBUG(SSSDBG_TRACE_FUNC, cr,
                             "Object found, but needs to be refreshed.\n");
             bypass_dp = false;
