@@ -430,6 +430,8 @@ static errno_t check_last_request(struct resp_ctx *rctx, const char *hint)
 struct get_domains_state {
     struct resp_ctx *rctx;
     struct sss_nc_ctx *optional_ncache;
+    get_domains_callback_fn_t *callback;
+    void *callback_pvt;
 };
 
 static void get_domains_at_startup_done(struct tevent_req *req)
@@ -462,6 +464,10 @@ static void get_domains_at_startup_done(struct tevent_req *req)
         }
     }
 
+    if (state->callback != NULL) {
+        state->callback(state->callback_pvt);
+    }
+
     talloc_free(state);
     return;
 }
@@ -489,7 +495,9 @@ static void get_domains_at_startup(struct tevent_context *ev,
 errno_t schedule_get_domains_task(TALLOC_CTX *mem_ctx,
                                   struct tevent_context *ev,
                                   struct resp_ctx *rctx,
-                                  struct sss_nc_ctx *optional_ncache)
+                                  struct sss_nc_ctx *optional_ncache,
+                                  get_domains_callback_fn_t *callback,
+                                  void *callback_pvt)
 {
     struct tevent_immediate *imm;
     struct get_domains_state *state;
@@ -500,6 +508,8 @@ errno_t schedule_get_domains_task(TALLOC_CTX *mem_ctx,
     }
     state->rctx = rctx;
     state->optional_ncache = optional_ncache;
+    state->callback = callback;
+    state->callback_pvt = callback_pvt;
 
     imm = tevent_create_immediate(mem_ctx);
     if (imm == NULL) {
@@ -630,7 +640,7 @@ static void sss_parse_inp_done(struct tevent_req *subreq)
                                      state->rawinp,
                                      &state->domname, &state->name);
     if (ret == EAGAIN && state->domname != NULL && state->name == NULL) {
-        DEBUG(SSSDBG_OP_FAILURE,
+        DEBUG(SSSDBG_FUNC_DATA,
               "Unknown domain in [%s]\n", state->rawinp);
         state->error = ERR_DOMAIN_NOT_FOUND;
     } else if (ret != EOK) {
@@ -684,6 +694,7 @@ struct tevent_req *
 sss_dp_get_account_domain_send(TALLOC_CTX *mem_ctx,
                                struct resp_ctx *rctx,
                                struct sss_domain_info *dom,
+                               bool fast_reply,
                                enum sss_dp_acct_type type,
                                uint32_t opt_id)
 {
@@ -693,6 +704,7 @@ sss_dp_get_account_domain_send(TALLOC_CTX *mem_ctx,
     struct be_conn *be_conn;
     uint32_t entry_type;
     char *filter;
+    uint32_t dp_flags;
     errno_t ret;
 
     req = tevent_req_create(mem_ctx, &state, struct sss_dp_get_account_domain_state);
@@ -733,10 +745,12 @@ sss_dp_get_account_domain_send(TALLOC_CTX *mem_ctx,
         goto done;
     }
 
+    dp_flags = fast_reply ? DP_FAST_REPLY : 0;
+
     subreq = sbus_call_dp_dp_getAccountDomain_send(state, be_conn->conn,
                                                    be_conn->bus_name,
-                                                   SSS_BUS_PATH, entry_type,
-                                                   filter);
+                                                   SSS_BUS_PATH, dp_flags,
+                                                   entry_type, filter);
     if (subreq == NULL) {
         DEBUG(SSSDBG_CRIT_FAILURE, "Unable to create subrequest!\n");
         ret = ENOMEM;
