@@ -38,7 +38,7 @@
 /* The maximum length of a request or reply as defined by the RPC
  * protocol. This is the same constant size as MIT KRB5 uses
  */
-#define KCM_PACKET_MAX_SIZE 2048
+#define KCM_PACKET_MAX_SIZE 10*1024*1024
 
 /* KCM operation, its raw input and raw output and result */
 struct kcm_op_io {
@@ -125,27 +125,30 @@ struct kcm_reqbuf {
     struct kcm_iovec v_len;
 
     /* Includes the major, minor versions etc */
-    uint8_t msgbuf[KCM_PACKET_MAX_SIZE];
     struct kcm_iovec v_msg;
 };
 
+static uint32_t kcm_input_get_payload_len(struct kcm_iovec *v)
+{
+    size_t lc = 0;
+    uint32_t len_be = 0;
+
+    /* The first 4 bytes before the payload is message length */
+    SAFEALIGN_COPY_UINT32_CHECK(&len_be, v->kiov_base, v->kiov_len, &lc);
+
+    return be32toh(len_be);
+}
+
 static errno_t kcm_input_parse(struct kcm_reqbuf *reqbuf,
                                struct kcm_op_io *op_io)
 {
-    size_t lc = 0;
     size_t mc = 0;
     uint16_t opcode_be = 0;
-    uint32_t len_be = 0;
     uint32_t msglen;
     uint8_t proto_maj = 0;
     uint8_t proto_min = 0;
 
-    /* The first 4 bytes before the payload is message length */
-    SAFEALIGN_COPY_UINT32_CHECK(&len_be,
-                                reqbuf->v_len.kiov_base,
-                                reqbuf->v_len.kiov_len,
-                                &lc);
-    msglen = be32toh(len_be);
+    msglen = kcm_input_get_payload_len(&reqbuf->v_len);
     DEBUG(SSSDBG_TRACE_LIBS,
           "Received message with length %"PRIu32"\n", msglen);
 
@@ -234,7 +237,6 @@ struct kcm_repbuf {
     uint8_t rcbuf[KCM_RETCODE_SIZE];
     struct kcm_iovec v_rc;
 
-    uint8_t msgbuf[KCM_PACKET_MAX_SIZE];
     struct kcm_iovec v_msg;
 };
 
@@ -255,11 +257,13 @@ static errno_t kcm_failbuf_construct(errno_t ret,
 /* retcode is 0 if the operation at least ran, non-zero if there
  * was some kind of internal KCM error, like input couldn't be parsed
  */
-static errno_t kcm_output_construct(struct kcm_op_io *op_io,
+static errno_t kcm_output_construct(TALLOC_CTX *mem_ctx,
+                                    struct kcm_op_io *op_io,
                                     struct kcm_repbuf *repbuf)
 {
-    size_t c;
+    uint8_t *rep;
     size_t replen;
+    size_t c;
 
     replen = sss_iobuf_get_len(op_io->reply);
     if (replen > KCM_PACKET_MAX_SIZE) {
@@ -277,14 +281,22 @@ static errno_t kcm_output_construct(struct kcm_op_io *op_io,
     SAFEALIGN_SETMEM_UINT32(repbuf->rcbuf, 0, &c);
 
     if (replen > 0) {
+        rep = talloc_zero_array(mem_ctx, uint8_t, replen);
+        if (rep == NULL) {
+            DEBUG(SSSDBG_CRIT_FAILURE,
+                  "Failed to allocate memory for the message\n");
+            return ENOMEM;
+        }
+
         c = 0;
-        SAFEALIGN_MEMCPY_CHECK(repbuf->msgbuf,
+        SAFEALIGN_MEMCPY_CHECK(rep,
                                sss_iobuf_get_data(op_io->reply),
                                replen,
-                               repbuf->v_msg.kiov_len,
+                               replen,
                                &c);
 
-        /* Length of the buffer to send to KCM client */
+        /* Set the buffer and its length to send to KCM client */
+        repbuf->v_msg.kiov_base = rep;
         repbuf->v_msg.kiov_len = replen;
     }
 
@@ -302,7 +314,7 @@ static void kcm_reply_error(struct cli_ctx *cctx,
     krb5_error_code kerr;
 
     DEBUG(SSSDBG_OP_FAILURE,
-          "KCM operation returs failure [%d]: %s\n",
+          "KCM operation returns failure [%d]: %s\n",
           retcode, sss_strerror(retcode));
     kerr = sss2krb5_error(retcode);
 
@@ -317,24 +329,6 @@ static void kcm_reply_error(struct cli_ctx *cctx,
     TEVENT_FD_WRITEABLE(cctx->cfde);
 }
 
-static void kcm_send_reply(struct cli_ctx *cctx,
-                           struct kcm_op_io *op_io,
-                           struct kcm_repbuf *repbuf)
-{
-    errno_t ret;
-
-    DEBUG(SSSDBG_TRACE_INTERNAL, "Sending a reply\n");
-    ret = kcm_output_construct(op_io, repbuf);
-    if (ret != EOK) {
-        DEBUG(SSSDBG_CRIT_FAILURE,
-              "Cannot construct the reply buffer, terminating client\n");
-        kcm_reply_error(cctx, ret, repbuf);
-        return;
-    }
-
-    TEVENT_FD_WRITEABLE(cctx->cfde);
-}
-
 /**
  * Request-reply dispatcher
  */
@@ -352,6 +346,26 @@ struct kcm_req_ctx {
     struct kcm_op_io op_io;
 };
 
+static void kcm_send_reply(struct kcm_req_ctx *req_ctx)
+{
+    struct cli_ctx *cctx;
+    errno_t ret;
+
+    DEBUG(SSSDBG_TRACE_INTERNAL, "Sending a reply\n");
+
+    cctx = req_ctx->cctx;
+
+    ret = kcm_output_construct(cctx, &req_ctx->op_io, &req_ctx->repbuf);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              "Cannot construct the reply buffer, terminating client\n");
+        kcm_reply_error(cctx, ret, &req_ctx->repbuf);
+        return;
+    }
+
+    TEVENT_FD_WRITEABLE(cctx->cfde);
+}
+
 static void kcm_cmd_request_done(struct tevent_req *req);
 
 static errno_t kcm_cmd_dispatch(struct kcm_ctx *kctx,
@@ -359,13 +373,16 @@ static errno_t kcm_cmd_dispatch(struct kcm_ctx *kctx,
 {
     struct tevent_req *req;
     struct cli_ctx *cctx;
+    struct kcm_conn_data *conn_data;
 
     cctx = req_ctx->cctx;
+    conn_data = talloc_get_type(cctx->state_ctx, struct kcm_conn_data);
 
     req = kcm_cmd_send(req_ctx,
                        cctx->ev,
                        kctx->qctx,
                        req_ctx->kctx->kcm_data,
+                       conn_data,
                        req_ctx->cctx->creds,
                        &req_ctx->op_io.request,
                        req_ctx->op_io.op);
@@ -381,11 +398,9 @@ static errno_t kcm_cmd_dispatch(struct kcm_ctx *kctx,
 static void kcm_cmd_request_done(struct tevent_req *req)
 {
     struct kcm_req_ctx *req_ctx;
-    struct cli_ctx *cctx;
     errno_t ret;
 
     req_ctx = tevent_req_callback_data(req, struct kcm_req_ctx);
-    cctx = req_ctx->cctx;
 
     ret = kcm_cmd_recv(req_ctx, req,
                        &req_ctx->op_io.reply);
@@ -393,15 +408,19 @@ static void kcm_cmd_request_done(struct tevent_req *req)
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE,
               "KCM operation failed [%d]: %s\n", ret, sss_strerror(ret));
-        kcm_reply_error(cctx, ret, &req_ctx->repbuf);
+        kcm_reply_error(req_ctx->cctx, ret, &req_ctx->repbuf);
         return;
     }
 
-    kcm_send_reply(cctx, &req_ctx->op_io, &req_ctx->repbuf);
+    kcm_send_reply(req_ctx);
 }
 
-static errno_t kcm_recv_data(int fd, struct kcm_reqbuf *reqbuf)
+static errno_t kcm_recv_data(TALLOC_CTX *mem_ctx,
+                             int fd,
+                             struct kcm_reqbuf *reqbuf)
 {
+    uint8_t *msg;
+    uint32_t msglen;
     errno_t ret;
 
     ret = kcm_read_iovec(fd, &reqbuf->v_len);
@@ -412,6 +431,24 @@ static errno_t kcm_recv_data(int fd, struct kcm_reqbuf *reqbuf)
         return ret;
     }
 
+    msglen = kcm_input_get_payload_len(&reqbuf->v_len);
+    if (msglen > KCM_PACKET_MAX_SIZE) {
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              "Request exceeds the KCM protocol limit, aborting\n");
+        return E2BIG;
+    }
+
+    msg = talloc_zero_array(mem_ctx, uint8_t, msglen);
+    if (msg == NULL) {
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              "Failed to allocate memory for the message\n");
+        return ENOMEM;
+    }
+
+    /* Set the buffer and its expected len to receive the data */
+    reqbuf->v_msg.kiov_base = msg;
+    reqbuf->v_msg.kiov_len = msglen;
+
     ret = kcm_read_iovec(fd, &reqbuf->v_msg);
     if (ret != EOK) {
         /* Not all errors are fatal, hence we don't print DEBUG messages
@@ -423,8 +460,10 @@ static errno_t kcm_recv_data(int fd, struct kcm_reqbuf *reqbuf)
     return EOK;
 }
 
-static struct kcm_req_ctx *kcm_new_req(TALLOC_CTX *mem_ctx,
-                                       struct cli_ctx *cctx,
+/* Mind that kcm_new_req() does not take a mem_ctx argument on purpose as we
+ * really want the cctx to be the memory context here so that if the client
+ * disconnects, the request goes away. */
+static struct kcm_req_ctx *kcm_new_req(struct cli_ctx *cctx,
                                        struct kcm_ctx *kctx)
 {
     struct kcm_req_ctx *req;
@@ -437,21 +476,12 @@ static struct kcm_req_ctx *kcm_new_req(TALLOC_CTX *mem_ctx,
     req->reqbuf.v_len.kiov_base = req->reqbuf.lenbuf;
     req->reqbuf.v_len.kiov_len = KCM_MSG_LEN_SIZE;
 
-    req->reqbuf.v_msg.kiov_base = req->reqbuf.msgbuf;
-    req->reqbuf.v_msg.kiov_len = KCM_PACKET_MAX_SIZE;
-
     req->repbuf.v_len.kiov_base = req->repbuf.lenbuf;
     req->repbuf.v_len.kiov_len = KCM_MSG_LEN_SIZE;
 
     req->repbuf.v_rc.kiov_base = req->repbuf.rcbuf;
     req->repbuf.v_rc.kiov_len = KCM_RETCODE_SIZE;
 
-    req->repbuf.v_msg.kiov_base = req->repbuf.msgbuf;
-    /* Length of the msg iobuf will be adjusted later, so far use the full
-     * length so that constructing the reply can use that capacity
-     */
-    req->repbuf.v_msg.kiov_len = KCM_PACKET_MAX_SIZE;
-
     req->cctx = cctx;
     req->kctx = kctx;
 
@@ -465,10 +495,10 @@ static void kcm_recv(struct cli_ctx *cctx)
     int ret;
 
     kctx = talloc_get_type(cctx->rctx->pvt_ctx, struct kcm_ctx);
-    req = talloc_get_type(cctx->state_ctx, struct kcm_req_ctx);
+    req = talloc_get_type(cctx->protocol_ctx, struct kcm_req_ctx);
     if (req == NULL) {
-        /* A new request comes in, setup data structures */
-        req = kcm_new_req(cctx, cctx, kctx);
+        /* A new request comes in, setup data structures. */
+        req = kcm_new_req(cctx, kctx);
         if (req == NULL) {
             DEBUG(SSSDBG_CRIT_FAILURE,
                   "Cannot set up client connection\n");
@@ -476,10 +506,20 @@ static void kcm_recv(struct cli_ctx *cctx)
             return;
         }
 
-        cctx->state_ctx = req;
+        cctx->protocol_ctx = req;
     }
 
-    ret = kcm_recv_data(cctx->cfd, &req->reqbuf);
+    /* Shared data between requests that originates in the same connection. */
+    if (cctx->state_ctx == NULL) {
+        cctx->state_ctx = talloc_zero(cctx, struct kcm_conn_data);
+        if (cctx->state_ctx == NULL) {
+            DEBUG(SSSDBG_CRIT_FAILURE, "Cannot set up client state\n");
+            talloc_free(cctx);
+            return;
+        }
+    }
+
+    ret = kcm_recv_data(req, cctx->cfd, &req->reqbuf);
     switch (ret) {
     case ENODATA:
         DEBUG(SSSDBG_TRACE_ALL, "Client closed connection.\n");
@@ -531,7 +571,7 @@ static int kcm_send_data(struct cli_ctx *cctx)
     struct kcm_req_ctx *req;
     errno_t ret;
 
-    req = talloc_get_type(cctx->state_ctx, struct kcm_req_ctx);
+    req = talloc_get_type(cctx->protocol_ctx, struct kcm_req_ctx);
 
     ret = kcm_write_iovec(cctx->cfd, &req->repbuf.v_len);
     if (ret != EOK) {
@@ -577,7 +617,7 @@ static void kcm_send(struct cli_ctx *cctx)
     DEBUG(SSSDBG_TRACE_INTERNAL, "All data sent!\n");
     TEVENT_FD_NOT_WRITEABLE(cctx->cfde);
     TEVENT_FD_READABLE(cctx->cfde);
-    talloc_zfree(cctx->state_ctx);
+    talloc_zfree(cctx->protocol_ctx);
     return;
 }
 
@@ -585,26 +625,7 @@ static void kcm_fd_handler(struct tevent_context *ev,
                            struct tevent_fd *fde,
                            uint16_t flags, void *ptr)
 {
-    errno_t ret;
-    struct cli_ctx *cctx = talloc_get_type(ptr, struct cli_ctx);
-
-    /* Always reset the idle timer on any activity */
-    ret = reset_client_idle_timer(cctx);
-    if (ret != EOK) {
-        DEBUG(SSSDBG_CRIT_FAILURE,
-              "Could not create idle timer for client. "
-              "This connection may not auto-terminate\n");
-        /* Non-fatal, continue */
-    }
-
-    if (flags & TEVENT_FD_READ) {
-        kcm_recv(cctx);
-        return;
-    }
-    if (flags & TEVENT_FD_WRITE) {
-        kcm_send(cctx);
-        return;
-    }
+    sss_client_fd_handler(ptr, kcm_recv, kcm_send, flags);
 }
 
 int kcm_connection_setup(struct cli_ctx *cctx)
