@@ -34,6 +34,7 @@
 
 #include "util/util.h"
 #include "util/sss_ptr_hash.h"
+#include "util/mmap_cache.h"
 #include "responder/nss/nss_private.h"
 #include "responder/nss/nss_iface.h"
 #include "responder/nss/nsssrv_mmap_cache.h"
@@ -57,21 +58,21 @@ nss_clear_memcache(TALLOC_CTX *mem_ctx,
     int memcache_timeout;
     errno_t ret;
 
-    ret = unlink(SSS_NSS_MCACHE_DIR"/"CLEAR_MC_FLAG);
-    if (ret != 0) {
+    if (access(SSS_NSS_MCACHE_DIR"/"CLEAR_MC_FLAG, F_OK) < 0) {
         ret = errno;
         if (ret == ENOENT) {
             DEBUG(SSSDBG_TRACE_FUNC,
                   "CLEAR_MC_FLAG not found. Nothing to do.\n");
             return ret;
         } else {
-            DEBUG(SSSDBG_CRIT_FAILURE, "Failed to unlink file: %s.\n",
+            DEBUG(SSSDBG_CRIT_FAILURE,
+                  "Failed to check existence of "CLEAR_MC_FLAG": %s.\n",
                   strerror(ret));
             return ret;
         }
     }
 
-    /* CLEAR_MC_FLAG removed successfully. Clearing memory caches. */
+    /* CLEAR_MC_FLAG found. Clearing memory caches. */
 
     ret = confdb_get_int(nctx->rctx->cdb,
                          CONFDB_NSS_CONF_ENTRY,
@@ -80,42 +81,47 @@ nss_clear_memcache(TALLOC_CTX *mem_ctx,
     if (ret != EOK) {
         DEBUG(SSSDBG_FATAL_FAILURE,
               "Unable to get memory cache entry timeout.\n");
-        return ret;
+        goto done;
     }
 
-    /* TODO: read cache sizes from configuration */
     DEBUG(SSSDBG_TRACE_FUNC, "Clearing memory caches.\n");
     ret = sss_mmap_cache_reinit(nctx, nctx->mc_uid, nctx->mc_gid,
-                                SSS_MC_CACHE_ELEMENTS,
+                                -1, /* keep current size */
                                 (time_t) memcache_timeout,
                                 &nctx->pwd_mc_ctx);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE,
               "passwd mmap cache invalidation failed\n");
-        return ret;
+        goto done;
     }
 
     ret = sss_mmap_cache_reinit(nctx, nctx->mc_uid, nctx->mc_gid,
-                                SSS_MC_CACHE_ELEMENTS,
+                                -1, /* keep current size */
                                 (time_t) memcache_timeout,
                                 &nctx->grp_mc_ctx);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE,
               "group mmap cache invalidation failed\n");
-        return ret;
+        goto done;
     }
 
     ret = sss_mmap_cache_reinit(nctx, nctx->mc_uid, nctx->mc_gid,
-                                SSS_MC_CACHE_ELEMENTS,
+                                -1, /* keep current size */
                                 (time_t)memcache_timeout,
                                 &nctx->initgr_mc_ctx);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE,
               "initgroups mmap cache invalidation failed\n");
-        return ret;
+        goto done;
     }
 
-    return EOK;
+done:
+    if (unlink(SSS_NSS_MCACHE_DIR"/"CLEAR_MC_FLAG) != 0) {
+        if (errno != ENOENT)
+            DEBUG(SSSDBG_CRIT_FAILURE, "Failed to unlink file: %s.\n",
+                  strerror(errno));
+    }
+    return ret;
 }
 
 static errno_t
@@ -210,8 +216,17 @@ done:
 
 static int setup_memcaches(struct nss_ctx *nctx)
 {
+    /* Default memcache sizes */
+    static const size_t SSS_MC_CACHE_SLOTS_PER_MB   = 1024*1024/MC_SLOT_SIZE;
+    static const size_t SSS_MC_CACHE_PASSWD_SIZE    =  8;
+    static const size_t SSS_MC_CACHE_GROUP_SIZE     =  6;
+    static const size_t SSS_MC_CACHE_INITGROUP_SIZE = 10;
+
     int ret;
     int memcache_timeout;
+    int mc_size_passwd;
+    int mc_size_group;
+    int mc_size_initgroups;
 
     /* Remove the CLEAR_MC_FLAG file if exists. */
     ret = unlink(SSS_NSS_MCACHE_DIR"/"CLEAR_MC_FLAG);
@@ -233,38 +248,80 @@ static int setup_memcaches(struct nss_ctx *nctx)
         return ret;
     }
 
-    if (memcache_timeout == 0) {
-        DEBUG(SSSDBG_CONF_SETTINGS,
-              "Fast in-memory cache will not be initialized.");
-        return EOK;
+    /* Get all memcache sizes from confdb (pwd, grp, initgr) */
+
+    ret = confdb_get_int(nctx->rctx->cdb,
+                         CONFDB_NSS_CONF_ENTRY,
+                         CONFDB_NSS_MEMCACHE_SIZE_PASSWD,
+                         SSS_MC_CACHE_PASSWD_SIZE,
+                         &mc_size_passwd);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_FATAL_FAILURE,
+              "Failed to get '"CONFDB_NSS_MEMCACHE_SIZE_PASSWD
+              "' option from confdb.\n");
+        return ret;
     }
 
-    /* TODO: read cache sizes from configuration */
+    ret = confdb_get_int(nctx->rctx->cdb,
+                         CONFDB_NSS_CONF_ENTRY,
+                         CONFDB_NSS_MEMCACHE_SIZE_GROUP,
+                         SSS_MC_CACHE_GROUP_SIZE,
+                         &mc_size_group);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_FATAL_FAILURE,
+              "Failed to get '"CONFDB_NSS_MEMCACHE_SIZE_GROUP
+              "' option from confdb.\n");
+        return ret;
+    }
+
+    ret = confdb_get_int(nctx->rctx->cdb,
+                         CONFDB_NSS_CONF_ENTRY,
+                         CONFDB_NSS_MEMCACHE_SIZE_INITGROUPS,
+                         SSS_MC_CACHE_INITGROUP_SIZE,
+                         &mc_size_initgroups);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_FATAL_FAILURE,
+              "Failed to get '"CONFDB_NSS_MEMCACHE_SIZE_INITGROUPS
+              "' option from confdb.\n");
+        return ret;
+    }
+
+    /* Initialize the fast in-memory caches if they were not disabled */
+
     ret = sss_mmap_cache_init(nctx, "passwd",
                               nctx->mc_uid, nctx->mc_gid,
                               SSS_MC_PASSWD,
-                              SSS_MC_CACHE_ELEMENTS, (time_t)memcache_timeout,
+                              mc_size_passwd * SSS_MC_CACHE_SLOTS_PER_MB,
+                              (time_t)memcache_timeout,
                               &nctx->pwd_mc_ctx);
     if (ret) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "passwd mmap cache is DISABLED\n");
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              "Failed to initialize passwd mmap cache: '%s'\n",
+              sss_strerror(ret));
     }
 
     ret = sss_mmap_cache_init(nctx, "group",
                               nctx->mc_uid, nctx->mc_gid,
                               SSS_MC_GROUP,
-                              SSS_MC_CACHE_ELEMENTS, (time_t)memcache_timeout,
+                              mc_size_group * SSS_MC_CACHE_SLOTS_PER_MB,
+                              (time_t)memcache_timeout,
                               &nctx->grp_mc_ctx);
     if (ret) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "group mmap cache is DISABLED\n");
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              "Failed to initialize group mmap cache: '%s'\n",
+              sss_strerror(ret));
     }
 
     ret = sss_mmap_cache_init(nctx, "initgroups",
                               nctx->mc_uid, nctx->mc_gid,
                               SSS_MC_INITGROUPS,
-                              SSS_MC_CACHE_ELEMENTS, (time_t)memcache_timeout,
+                              mc_size_initgroups * SSS_MC_CACHE_SLOTS_PER_MB,
+                              (time_t)memcache_timeout,
                               &nctx->initgr_mc_ctx);
     if (ret) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "initgroups mmap cache is DISABLED\n");
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              "Failed to initialize initgroups mmap cache: '%s'\n",
+              sss_strerror(ret));
     }
 
     return EOK;
@@ -290,7 +347,7 @@ nss_register_service_iface(struct nss_ctx *nss_ctx,
 
     ret = sbus_connection_add_path(rctx->mon_conn, SSS_BUS_PATH, &iface_svc);
     if (ret != EOK) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "Unable to register service interface"
+        DEBUG(SSSDBG_FATAL_FAILURE, "Unable to register service interface"
               "[%d]: %s\n", ret, sss_strerror(ret));
     }
 
@@ -337,23 +394,23 @@ static int sssd_supplementary_group(struct nss_ctx *nss_ctx)
             ret = ENOMEM;
             goto done;
         }
-    }
 
-    size = getgroups(size, supp_gids);
-    if (size == -1) {
-        ret = errno;
-        DEBUG(SSSDBG_CRIT_FAILURE, "Getgroups failed! (%d, %s)\n",
-                                    ret, sss_strerror(ret));
-        goto done;
-    }
-
-    for (int i = 0; i < size; i++) {
-        if (supp_gids[i] == nss_ctx->mc_gid) {
-            DEBUG(SSSDBG_TRACE_FUNC,
-                  "Already assigned to the SSSD supplementary group\n");
-            ret = EOK;
+        size = getgroups(size, supp_gids);
+        if (size == -1) {
+            ret = errno;
+            DEBUG(SSSDBG_CRIT_FAILURE, "Getgroups failed! (%d, %s)\n",
+                                        ret, sss_strerror(ret));
             goto done;
         }
+
+        for (int i = 0; i < size; i++) {
+            if (supp_gids[i] == nss_ctx->mc_gid) {
+                DEBUG(SSSDBG_TRACE_FUNC,
+                      "Already assigned to the SSSD supplementary group\n");
+                ret = EOK;
+                goto done;
+            }
+        }
     }
 
     ret = setgroups(1, &nss_ctx->mc_gid);
@@ -454,6 +511,21 @@ int nss_process_init(TALLOC_CTX *mem_ctx,
         ret = EFAULT;
         goto fail;
     }
+
+    nctx->hostent = talloc_zero(nctx, struct nss_enum_ctx);
+    if (nctx->hostent == NULL) {
+        DEBUG(SSSDBG_FATAL_FAILURE, "Unable to initialize hostent context!\n");
+        ret = ENOMEM;
+        goto fail;
+    }
+
+    nctx->netent = talloc_zero(nctx, struct nss_enum_ctx);
+    if (nctx->netent == NULL) {
+        DEBUG(SSSDBG_FATAL_FAILURE, "Unable to initialize netent context!\n");
+        ret = ENOMEM;
+        goto fail;
+    }
+
     /*
      * Adding the NSS process to the SSSD supplementary group avoids
      * dac_override AVC messages from SELinux in case sssd_nss runs
@@ -485,7 +557,8 @@ int nss_process_init(TALLOC_CTX *mem_ctx,
     }
     responder_set_fd_limit(fd_limit);
 
-    ret = schedule_get_domains_task(rctx, rctx->ev, rctx, nctx->rctx->ncache);
+    ret = schedule_get_domains_task(rctx, rctx->ev, rctx, nctx->rctx->ncache,
+                                    NULL, NULL);
     if (ret != EOK) {
         DEBUG(SSSDBG_FATAL_FAILURE, "schedule_get_domains_tasks failed.\n");
         goto fail;
@@ -559,7 +632,7 @@ int main(int argc, const char *argv[])
 
     sss_set_logger(opt_logger);
 
-    ret = server_setup("sssd[nss]", 0, uid, gid, CONFDB_NSS_CONF_ENTRY,
+    ret = server_setup("nss", 0, uid, gid, CONFDB_NSS_CONF_ENTRY,
                        &main_ctx);
     if (ret != EOK) return 2;
 
