@@ -138,7 +138,7 @@ static void inform_user(struct pam_data* pd, const char *pam_message)
     ret = pack_user_info_msg(pd, pam_message, &msg_len, &msg);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE,
-              "pack_user_info_account_expired failed.\n");
+              "pack_user_info_msg failed.\n");
     } else {
         ret = pam_add_response(pd, SSS_PAM_USER_INFO, msg_len, msg);
         if (ret != EOK) {
@@ -752,9 +752,6 @@ static int pam_reply_sr_export_shell(struct pam_auth_req *preq,
     if (preq->cctx->rctx->sr_conf.scope ==
             SESSION_RECORDING_SCOPE_NONE) {
         enabled = false;
-    } else if (preq->cctx->rctx->sr_conf.scope ==
-            SESSION_RECORDING_SCOPE_ALL) {
-        enabled = true;
     } else {
         enabled_str = ldb_msg_find_attr_as_string(preq->user_obj,
                                                   SYSDB_SESSION_RECORDING, NULL);
@@ -982,6 +979,7 @@ static void pam_reply(struct pam_auth_req *preq)
     /* If this was a successful login, save the lastLogin time */
     if (pd->cmd == SSS_PAM_AUTHENTICATE &&
         pd->pam_status == PAM_SUCCESS &&
+        preq->domain &&
         preq->domain->cache_credentials &&
         !pd->offline_auth &&
         !pd->last_auth_saved &&
@@ -1260,7 +1258,7 @@ static errno_t pam_forwarder_parse_data(struct cli_ctx *cctx, struct pam_data *p
                     || sss_authtok_get_type(pd->authtok)
                                                == SSS_AUTHTOK_TYPE_SC_KEYPAD)) {
             ret = sss_authtok_get_sc(pd->authtok, NULL, NULL, NULL, NULL, NULL,
-                                     NULL, &key_id, NULL);
+                                     NULL, &key_id, NULL, NULL, NULL);
             if (ret != EOK) {
                 DEBUG(SSSDBG_OP_FAILURE, "sss_authtok_get_sc failed.\n");
                 goto done;
@@ -1405,7 +1403,7 @@ static errno_t check_cert(TALLOC_CTX *mctx,
     }
 
     req = pam_check_cert_send(mctx, ev,
-                              pctx->nss_db, p11_child_timeout,
+                              pctx->ca_db, p11_child_timeout,
                               cert_verification_opts, pctx->sss_certmap_ctx,
                               uri, pd);
     if (req == NULL) {
@@ -1905,6 +1903,7 @@ static int pam_check_user_search(struct pam_auth_req *preq)
 
     cache_req_data_set_bypass_cache(data, false);
     cache_req_data_set_bypass_dp(data, true);
+    cache_req_data_set_requested_domains(data, preq->pd->requested_domains);
 
     dpreq = cache_req_send(preq,
                            preq->cctx->rctx->ev,
@@ -1942,7 +1941,7 @@ static void pam_check_user_search_next(struct tevent_req *req)
     talloc_zfree(req);
     if (ret != EOK && ret != ENOENT) {
         DEBUG(SSSDBG_OP_FAILURE, "Cache lookup failed, trying to get fresh "
-                                 "data from the backened.\n");
+                                 "data from the backend.\n");
     }
 
     DEBUG(SSSDBG_TRACE_ALL, "PAM initgroups scheme [%s].\n",
@@ -2013,6 +2012,7 @@ static void pam_check_user_search_next(struct tevent_req *req)
     }
     cache_req_data_set_bypass_cache(data, true);
     cache_req_data_set_bypass_dp(data, false);
+    cache_req_data_set_requested_domains(data, preq->pd->requested_domains);
 
     dpreq = cache_req_send(preq,
                            preq->cctx->rctx->ev,
@@ -2274,7 +2274,8 @@ static void pam_dom_forwarder(struct pam_auth_req *preq)
                                  SSS_AUTHTOK_TYPE_SC_PIN, NULL, 0,
                                  sss_cai_get_token_name(preq->current_cert), 0,
                                  sss_cai_get_module_name(preq->current_cert), 0,
-                                 sss_cai_get_key_id(preq->current_cert), 0);
+                                 sss_cai_get_key_id(preq->current_cert), 0,
+                                 sss_cai_get_label(preq->current_cert), 0);
                         if (ret != EOK) {
                             DEBUG(SSSDBG_OP_FAILURE,
                                   "sss_authtok_set_sc failed, Smartcard "
@@ -2401,6 +2402,8 @@ struct sss_cmd_table *get_pam_cmds(void)
         {SSS_PAM_CHAUTHTOK, pam_cmd_chauthtok},
         {SSS_PAM_CHAUTHTOK_PRELIM, pam_cmd_chauthtok_prelim},
         {SSS_PAM_PREAUTH, pam_cmd_preauth},
+        {SSS_GSSAPI_INIT, pam_cmd_gssapi_init},
+        {SSS_GSSAPI_SEC_CTX, pam_cmd_gssapi_sec_ctx},
         {SSS_CLI_NULL, NULL}
     };
 
