@@ -585,26 +585,7 @@ static void sec_fd_handler(struct tevent_context *ev,
                            struct tevent_fd *fde,
                            uint16_t flags, void *ptr)
 {
-    errno_t ret;
-    struct cli_ctx *cctx = talloc_get_type(ptr, struct cli_ctx);
-
-    /* Always reset the idle timer on any activity */
-    ret = reset_client_idle_timer(cctx);
-    if (ret != EOK) {
-        DEBUG(SSSDBG_CRIT_FAILURE,
-              "Could not create idle timer for client. "
-               "This connection may not auto-terminate\n");
-        /* Non-fatal, continue */
-    }
-
-    if (flags & TEVENT_FD_READ) {
-        sec_recv(cctx);
-        return;
-    }
-    if (flags & TEVENT_FD_WRITE) {
-        sec_send(cctx);
-        return;
-    }
+    sss_client_fd_handler(ptr, sec_recv, sec_send, flags);
 }
 
 static http_parser_settings sec_callbacks = {
