@@ -83,8 +83,9 @@ static int sudosrv_response_append_attr(TALLOC_CTX *mem_ctx,
     uint8_t *response_body = *_response_body;
     size_t response_len = *_response_len;
     TALLOC_CTX *tmp_ctx = NULL;
-    int i = 0;
+    unsigned int i = 0;
     int ret = EOK;
+    const char *strval;
 
     tmp_ctx = talloc_new(NULL);
     if (tmp_ctx == NULL) {
@@ -108,16 +109,19 @@ static int sudosrv_response_append_attr(TALLOC_CTX *mem_ctx,
 
     /* values */
     for (i = 0; i < values_num; i++) {
-        if (strlen((char*)(values[i].data)) != values[i].length) {
+        strval = (const char *) values[i].data;
+
+        if (strlen((strval)) != values[i].length) {
             DEBUG(SSSDBG_CRIT_FAILURE, "value is not a string\n");
             ret = EINVAL;
             goto done;
         }
 
         ret = sudosrv_response_append_string(tmp_ctx,
-                                             (const char*)values[i].data,
+                                             strval,
                                              values[i].length + 1,
                                              &response_body, &response_len);
+        DEBUG(SSSDBG_TRACE_INTERNAL, "%s:%s\n", name, strval);
         if (ret != EOK) {
             goto done;
         }
@@ -211,6 +215,7 @@ errno_t sudosrv_build_response(TALLOC_CTX *mem_ctx,
     if (ret != EOK) {
         goto fail;
     }
+    DEBUG(SSSDBG_TRACE_INTERNAL, "error: [%"PRIu32"]\n", error);
 
     if (error != SSS_SUDO_ERROR_OK) {
         goto done;
@@ -230,9 +235,11 @@ errno_t sudosrv_build_response(TALLOC_CTX *mem_ctx,
     if (ret != EOK) {
         goto fail;
     }
+    DEBUG(SSSDBG_TRACE_INTERNAL, "rules_num: [%"PRIu32"]\n", error);
 
     /* rules */
     for (i = 0; i < rules_num; i++) {
+        DEBUG(SSSDBG_TRACE_INTERNAL, "rule [%"PRIu32"]/[%"PRIu32"]\n", i+1, rules_num);
         ret = sudosrv_response_append_rule(tmp_ctx, rules[i]->num, rules[i]->a,
                                            &response_body, &response_len);
         if (ret != EOK) {
@@ -251,179 +258,50 @@ fail:
     return ret;
 }
 
-struct sudosrv_parse_query_state {
-    struct resp_ctx *rctx;
-    uid_t uid;
-    char *rawname;
-};
-
-static void sudosrv_parse_query_done(struct tevent_req *subreq);
-
-struct tevent_req *sudosrv_parse_query_send(TALLOC_CTX *mem_ctx,
-                                            struct resp_ctx *rctx,
-                                            uint8_t *query_body,
-                                            size_t query_len)
+errno_t sudosrv_parse_query(TALLOC_CTX *mem_ctx,
+                            uint8_t *query_body,
+                            size_t query_len,
+                            char **_rawname,
+                            uid_t *_uid)
 {
-    struct tevent_req *req = NULL;
-    struct tevent_req *subreq = NULL;
-    struct sudosrv_parse_query_state *state = NULL;
     size_t offset = 0;
-    size_t rawname_len = 0;
-    char *rawname = NULL;
-    char *domainname = NULL;
-    errno_t ret;
-
-    /* create request */
-    req = tevent_req_create(mem_ctx, &state,
-                            struct sudosrv_parse_query_state);
-    if (req == NULL) {
-        DEBUG(SSSDBG_FATAL_FAILURE, "tevent_req_create() failed\n");
-        return NULL;
-    }
-
-    state->rctx = rctx;
+    size_t rawname_len;
+    char *rawname;
+    uid_t uid;
 
     /* uid */
-
     if (query_len < sizeof(uid_t)) {
         DEBUG(SSSDBG_CRIT_FAILURE, "Query is too small\n");
-        ret = EINVAL;
-        goto done;
+        return EINVAL;
     }
-    safealign_memcpy(&state->uid, query_body, sizeof(uid_t), &offset);
+    safealign_memcpy(&uid, query_body, sizeof(uid_t), &offset);
 
     /* username[@domain] */
-
     rawname = (char*)(query_body + offset);
     rawname_len = query_len - offset; /* strlen + zero */
 
     if (rawname[rawname_len - 1] != '\0') {
         DEBUG(SSSDBG_CRIT_FAILURE, "Username is not zero terminated\n");
-        ret = EINVAL;
-        goto done;
+        return EINVAL;
     }
 
     if (rawname_len < 2) { /* at least one character and zero */
         DEBUG(SSSDBG_CRIT_FAILURE, "Query does not contain username\n");
-        ret = EINVAL;
-        goto done;
+        return EINVAL;
     }
 
     if (!sss_utf8_check((uint8_t*)rawname, rawname_len - 1)) {
         DEBUG(SSSDBG_CRIT_FAILURE, "Supplied data is not valid UTF-8 string\n");
-        ret = EINVAL;
-        goto done;
-    }
-
-    /* parse username */
-
-    state->rawname = rawname;
-    ret = sss_parse_name_for_domains(state, rctx->domains,
-                                     rctx->default_domain, state->rawname,
-                                     &domainname, NULL);
-    if (ret == EAGAIN) {
-        DEBUG(SSSDBG_TRACE_FUNC, "Domain [%s] not found, "
-              "sending subdomain request\n", domainname);
-
-        subreq = sss_dp_get_domains_send(state, rctx, true, domainname);
-        if (subreq == NULL) {
-            ret = ENOMEM;
-        } else {
-            tevent_req_set_callback(subreq, sudosrv_parse_query_done, req);
-            ret = EAGAIN;
-        }
-        goto done;
-    } else if (ret != EOK) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "Invalid name received [%s]\n", rawname);
-        goto done;
-    }
-
-    ret = EOK;
-
-done:
-    if (ret != EAGAIN) {
-        if (ret == EOK) {
-            tevent_req_done(req);
-        } else {
-            tevent_req_error(req, ret);
-        }
-        tevent_req_post(req, rctx->ev);
-    }
-
-    return req;
-}
-
-static void sudosrv_parse_query_done(struct tevent_req *subreq)
-{
-    struct tevent_req *req = NULL;
-    errno_t ret;
-
-    req = tevent_req_callback_data(subreq, struct tevent_req);
-
-    ret = sss_dp_get_domains_recv(subreq);
-    talloc_free(subreq);
-    if (ret != EOK) {
-        tevent_req_error(req, ret);
-        return;
-    }
-
-    tevent_req_done(req);
-}
-
-errno_t sudosrv_parse_query_recv(TALLOC_CTX *mem_ctx,
-                                 struct tevent_req *req,
-                                 uid_t *_uid,
-                                 char **_username,
-                                 struct sss_domain_info **_domain)
-{
-    struct sudosrv_parse_query_state *state = NULL;
-    struct sss_domain_info *domain = NULL;
-    char *username = NULL;
-    char *domainname = NULL;
-    errno_t ret;
-
-    state = tevent_req_data(req, struct sudosrv_parse_query_state);
-
-    TEVENT_REQ_RETURN_ON_ERROR(req);
-
-    if (state->rawname == NULL) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "No query specified?!\n");
         return EINVAL;
     }
 
-    /* Try to parse username@domain again because if the first call
-     * returned EAGAIN, then username is unset. If we get EAGAIN again,
-     * we will not search for it again.
-     */
-    ret = sss_parse_name_for_domains(state, state->rctx->domains,
-                                     state->rctx->default_domain,
-                                     state->rawname,
-                                     &domainname, &username);
-    if (ret != EOK) {
-        DEBUG(SSSDBG_TRACE_FUNC, "Unable to parse domain [%d]: %s\n",
-                                  ret, strerror(ret));
-        return ret;
+    rawname = talloc_strdup(mem_ctx, rawname);
+    if (rawname == NULL) {
+        return ENOMEM;
     }
 
-    if (username == NULL) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "No username specified!\n");
-        return EINVAL;
-    }
-
-    if (domainname != NULL) {
-        /* mem_ctx because it duplicates only subdomains not domains
-         * so I cannot easily steal it */
-        domain = responder_get_domain(state->rctx, domainname);
-        if (domain == NULL) {
-            DEBUG(SSSDBG_OP_FAILURE, "Corresponding domain [%s] has not been "
-                                      "found\n", domainname);
-            return ENOENT;
-        }
-    }
-
-    *_uid = state->uid;
-    *_username = talloc_steal(mem_ctx, username);
-    *_domain = domain; /* do not steal on mem_ctx */
+    *_uid = uid;
+    *_rawname = rawname;
 
     return EOK;
 }
