@@ -45,10 +45,14 @@ int main(int argc, const char *argv[])
     char *value = NULL;
     char *pc_key = NULL;
     int pc_setent = 0;
+    int pc_protocol = 1;
+    unsigned int protocol;
+    unsigned int requested_protocol = 1;
     struct poptOption long_options[] = {
         POPT_AUTOHELP
         { "by-name",  'n', POPT_ARG_STRING, &pc_key, 0, "Request map by name", NULL },
         { "only-setent",  's', POPT_ARG_VAL, &pc_setent, 1, "Run only setent, do not enumerate", NULL },
+        { "protocol",  'p', POPT_ARG_INT, &pc_protocol, 0, "Protocol version", NULL },
         POPT_TABLEEND
     };
     poptContext pc = NULL;
@@ -69,6 +73,14 @@ int main(int argc, const char *argv[])
 
     poptFreeContext(pc);
 
+    requested_protocol = pc_protocol;
+    protocol = _sss_auto_protocol_version(requested_protocol);
+    if (protocol != requested_protocol) {
+        fprintf(stderr, "Unsupported protocol version: %d -> %d\n",
+                requested_protocol, protocol);
+        exit(EXIT_FAILURE);
+    }
+
     ret = _sss_setautomntent(mapname, &ctx);
     if (ret) {
         fprintf(stderr, "setautomntent failed [%d]: %s\n",
