@@ -73,9 +73,12 @@ const char *wbcErrorString(wbcErr error);
  *  0.10: Added wbcPingDc2()
  *  0.11: Extended wbcAuthenticateUserEx to provide PAC parsing
  *  0.12: Added wbcCtxCreate and friends
+ *  0.13: Added wbcCtxUnixIdsToSids and wbcUnixIdsToSids
+ *  0.14: Added "authoritative" to wbcAuthErrorInfo
+ *        Added WBC_SID_NAME_LABEL
  **/
 #define WBCLIENT_MAJOR_VERSION 0
-#define WBCLIENT_MINOR_VERSION 12
+#define WBCLIENT_MINOR_VERSION 14
 #define WBCLIENT_VENDOR_VERSION "Samba libwbclient"
 struct wbcLibraryDetails {
     uint16_t major_version;
@@ -137,7 +140,8 @@ enum wbcSidType {
     WBC_SID_NAME_DELETED=6,
     WBC_SID_NAME_INVALID=7,
     WBC_SID_NAME_UNKNOWN=8,
-    WBC_SID_NAME_COMPUTER=9
+    WBC_SID_NAME_COMPUTER=9,
+    WBC_SID_NAME_LABEL=10
 };
 
 /**
@@ -315,6 +319,7 @@ struct wbcChangePasswordParams {
 #define WBC_MSV1_0_ALLOW_SERVER_TRUST_ACCOUNT         0x00000020
 #define WBC_MSV1_0_RETURN_PROFILE_PATH                0x00000200
 #define WBC_MSV1_0_ALLOW_WORKSTATION_TRUST_ACCOUNT    0x00000800
+#define WBC_MSV1_0_ALLOW_MSVCHAPV2                    0x00010000
 
 /* wbcAuthUserParams->flags */
 
@@ -417,6 +422,7 @@ struct wbcAuthErrorInfo {
     char *nt_string;
     int32_t pam_error;
     char *display_string;
+    uint8_t authoritative;
 };
 
 /**
@@ -1030,6 +1036,34 @@ wbcErr wbcCtxSidsToUnixIds(struct wbcContext *ctx,
 wbcErr wbcSidsToUnixIds(const struct wbcDomainSid *sids, uint32_t num_sids,
                         struct wbcUnixId *ids);
 
+/**
+ * @brief Convert a list of unix ids to sids
+ *
+ * @param ctx         wbclient Context
+ * @param ids         Pointer to an array of UNIX IDs to convert
+ * @param num_ids     Number of UNIX IDs
+ * @param sids        Preallocated output array for translated SIDs
+ *
+ * @return #wbcErr
+ *
+ **/
+wbcErr wbcCtxUnixIdsToSids(struct wbcContext *ctx,
+                           const struct wbcUnixId *ids, uint32_t num_ids,
+                           struct wbcDomainSid *sids);
+
+/**
+ * @brief Convert a list of unix ids to sids
+ *
+ * @param ids         Pointer to an array of UNIX IDs to convert
+ * @param num_ids     Number of UNIX IDs
+ * @param sids        Preallocated output array for translated SIDs
+ *
+ * @return #wbcErr
+ *
+ **/
+wbcErr wbcUnixIdsToSids(const struct wbcUnixId *ids, uint32_t num_ids,
+                        struct wbcDomainSid *sids);
+
 /**
  * @brief Obtain a new uid from Winbind
  *
@@ -1076,7 +1110,7 @@ wbcErr wbcAllocateGid(gid_t *pgid);
  *
  * @return #wbcErr
  *
- * @deprecated      This method is not impemented any more and should
+ * @deprecated      This method is not implemented any more and should
  *                  be removed in the next major version change.
  **/
 wbcErr wbcSetUidMapping(uid_t uid, const struct wbcDomainSid *sid);
@@ -1593,7 +1627,7 @@ wbcErr wbcLookupDomainControllerEx(const char *domain,
                                    struct wbcDomainControllerInfoEx **dc_info);
 
 /**********************************************************
- * Athenticate functions
+ * Authenticate functions
  **********************************************************/
 
 /**
