@@ -28,6 +28,7 @@
 #include <string.h>
 #include "sss_cli.h"
 #include "nss_mc.h"
+#include "nss_common.h"
 
 static struct sss_nss_getpwent_data {
     size_t len;
@@ -63,14 +64,8 @@ static void sss_nss_getpwent_data_clean(void) {
  *  8-X: sequence of 5, 0 terminated, strings (name, passwd, gecos, dir, shell)
  */
 
-struct sss_nss_pw_rep {
-    struct passwd *result;
-    char *buffer;
-    size_t buflen;
-};
-
-static int sss_nss_getpw_readrep(struct sss_nss_pw_rep *pr,
-                                 uint8_t *buf, size_t *len)
+int sss_nss_getpw_readrep(struct sss_nss_pw_rep *pr,
+                          uint8_t *buf, size_t *len)
 {
     errno_t ret;
     size_t i, slen, dlen;
@@ -139,6 +134,7 @@ enum nss_status _nss_sss_getpwnam_r(const char *name, struct passwd *result,
     struct sss_nss_pw_rep pwrep;
     uint8_t *repbuf;
     size_t replen, len, name_len;
+    uint32_t num_results;
     enum nss_status nret;
     int ret;
 
@@ -167,7 +163,7 @@ enum nss_status _nss_sss_getpwnam_r(const char *name, struct passwd *result,
          * if no entry is found */
         break;
     default:
-        /* if using the mmaped cache failed,
+        /* if using the mmapped cache failed,
          * fall back to socket based comms */
         break;
     }
@@ -177,6 +173,27 @@ enum nss_status _nss_sss_getpwnam_r(const char *name, struct passwd *result,
 
     sss_nss_lock();
 
+    /* previous thread might already initialize entry in mmap cache */
+    ret = sss_nss_mc_getpwnam(name, name_len, result, buffer, buflen);
+    switch (ret) {
+    case 0:
+        *errnop = 0;
+        nret = NSS_STATUS_SUCCESS;
+        goto out;
+    case ERANGE:
+        *errnop = ERANGE;
+        nret = NSS_STATUS_TRYAGAIN;
+        goto out;
+    case ENOENT:
+        /* fall through, we need to actively ask the parent
+         * if no entry is found */
+        break;
+    default:
+        /* if using the mmapped cache failed,
+         * fall back to socket based comms */
+        break;
+    }
+
     nret = sss_nss_make_request(SSS_NSS_GETPWNAM, &rd,
                                 &repbuf, &replen, errnop);
     if (nret != NSS_STATUS_SUCCESS) {
@@ -187,15 +204,18 @@ enum nss_status _nss_sss_getpwnam_r(const char *name, struct passwd *result,
     pwrep.buffer = buffer;
     pwrep.buflen = buflen;
 
+    /* Get number of results from repbuf. */
+    SAFEALIGN_COPY_UINT32(&num_results, repbuf, NULL);
+
     /* no results if not found */
-    if (((uint32_t *)repbuf)[0] == 0) {
+    if (num_results == 0) {
         free(repbuf);
         nret = NSS_STATUS_NOTFOUND;
         goto out;
     }
 
     /* only 1 result is accepted for this function */
-    if (((uint32_t *)repbuf)[0] != 1) {
+    if (num_results != 1) {
         *errnop = EBADMSG;
         free(repbuf);
         nret = NSS_STATUS_TRYAGAIN;
@@ -225,12 +245,16 @@ enum nss_status _nss_sss_getpwuid_r(uid_t uid, struct passwd *result,
     struct sss_nss_pw_rep pwrep;
     uint8_t *repbuf;
     size_t replen, len;
+    uint32_t num_results;
     enum nss_status nret;
     uint32_t user_uid;
     int ret;
 
     /* Caught once glibc passing in buffer == 0x0 */
-    if (!buffer || !buflen) return ERANGE;
+    if (!buffer || !buflen) {
+	*errnop = ERANGE;
+	return NSS_STATUS_TRYAGAIN;
+    }
 
     ret = sss_nss_mc_getpwuid(uid, result, buffer, buflen);
     switch (ret) {
@@ -245,7 +269,7 @@ enum nss_status _nss_sss_getpwuid_r(uid_t uid, struct passwd *result,
          * if no entry is found */
         break;
     default:
-        /* if using the mmaped cache failed,
+        /* if using the mmapped cache failed,
          * fall back to socket based comms */
         break;
     }
@@ -256,6 +280,27 @@ enum nss_status _nss_sss_getpwuid_r(uid_t uid, struct passwd *result,
 
     sss_nss_lock();
 
+    /* previous thread might already initialize entry in mmap cache */
+    ret = sss_nss_mc_getpwuid(uid, result, buffer, buflen);
+    switch (ret) {
+    case 0:
+        *errnop = 0;
+        nret = NSS_STATUS_SUCCESS;
+        goto out;
+    case ERANGE:
+        *errnop = ERANGE;
+        nret = NSS_STATUS_TRYAGAIN;
+        goto out;
+    case ENOENT:
+        /* fall through, we need to actively ask the parent
+         * if no entry is found */
+        break;
+    default:
+        /* if using the mmapped cache failed,
+         * fall back to socket based comms */
+        break;
+    }
+
     nret = sss_nss_make_request(SSS_NSS_GETPWUID, &rd,
                                 &repbuf, &replen, errnop);
     if (nret != NSS_STATUS_SUCCESS) {
@@ -266,15 +311,18 @@ enum nss_status _nss_sss_getpwuid_r(uid_t uid, struct passwd *result,
     pwrep.buffer = buffer;
     pwrep.buflen = buflen;
 
+    /* Get number of results from repbuf. */
+    SAFEALIGN_COPY_UINT32(&num_results, repbuf, NULL);
+
     /* no results if not found */
-    if (((uint32_t *)repbuf)[0] == 0) {
+    if (num_results == 0) {
         free(repbuf);
         nret = NSS_STATUS_NOTFOUND;
         goto out;
     }
 
     /* only 1 result is accepted for this function */
-    if (((uint32_t *)repbuf)[0] != 1) {
+    if (num_results != 1) {
         *errnop = EBADMSG;
         free(repbuf);
         nret = NSS_STATUS_TRYAGAIN;
@@ -325,12 +373,16 @@ static enum nss_status internal_getpwent_r(struct passwd *result,
     struct sss_nss_pw_rep pwrep;
     uint8_t *repbuf;
     size_t replen;
+    uint32_t num_results;
     enum nss_status nret;
     uint32_t num_entries;
     int ret;
 
     /* Caught once glibc passing in buffer == 0x0 */
-    if (!buffer || !buflen) return ERANGE;
+    if (!buffer || !buflen) {
+	*errnop = ERANGE;
+	return NSS_STATUS_TRYAGAIN;
+    }
 
     /* if there are leftovers return the next one */
     if (sss_nss_getpwent_data.data != NULL &&
@@ -369,8 +421,11 @@ static enum nss_status internal_getpwent_r(struct passwd *result,
         return nret;
     }
 
+    /* Get number of results from repbuf. */
+    SAFEALIGN_COPY_UINT32(&num_results, repbuf, NULL);
+
     /* no results if not found */
-    if ((((uint32_t *)repbuf)[0] == 0) || (replen - 8 == 0)) {
+    if ((num_results == 0) || (replen - 8 == 0)) {
         free(repbuf);
         return NSS_STATUS_NOTFOUND;
     }
@@ -400,6 +455,7 @@ enum nss_status _nss_sss_endpwent(void)
 {
     enum nss_status nret;
     int errnop;
+    int saved_errno = errno;
 
     sss_nss_lock();
 
@@ -410,6 +466,8 @@ enum nss_status _nss_sss_endpwent(void)
                                 NULL, NULL, NULL, &errnop);
     if (nret != NSS_STATUS_SUCCESS) {
         errno = errnop;
+    } else {
+        errno = saved_errno;
     }
 
     sss_nss_unlock();
