@@ -48,7 +48,7 @@ void setup_check_and_open(void)
     filename = strdup(FILENAME_TEMPLATE);
     fail_unless(filename != NULL, "strdup failed");
 
-    old_umask = umask(SSS_DFL_X_UMASK);
+    old_umask = umask(SSS_DFL_UMASK);
     ret = mkstemp(filename);
     umask(old_umask);
     fail_unless(ret != -1, "mkstemp failed [%d][%s]", errno, strerror(errno));
@@ -99,7 +99,7 @@ START_TEST(test_symlink)
 
     ret = snprintf(newpath, newpath_length, "%s%s", filename, SUFFIX);
     fail_unless(ret == newpath_length - 1,
-                "snprintf failed: expected [%d] got [%d]", newpath_length -1,
+                "snprintf failed: expected [%zd] got [%d]", newpath_length - 1,
                                                            ret);
 
     ret = symlink(filename, newpath);
@@ -126,7 +126,7 @@ START_TEST(test_follow_symlink)
 
     ret = snprintf(newpath, newpath_length, "%s%s", filename, SUFFIX);
     fail_unless(ret == newpath_length - 1,
-                "snprintf failed: expected [%d] got [%d]", newpath_length -1,
+                "snprintf failed: expected [%zd] got [%d]", newpath_length - 1,
                                                            ret);
 
     ret = symlink(filename, newpath);
