@@ -31,5 +31,13 @@ void test_replace_whitespaces(void **state);
 void test_reverse_replace_whitespaces(void **state);
 void test_guid_blob_to_string_buf(void **state);
 void test_get_last_x_chars(void **state);
+void test_concatenate_string_array(void **state);
+
+/* from src/tests/cmocka/test_sss_ptr_hash.c */
+void test_sss_ptr_hash_with_free_cb(void **state);
+void test_sss_ptr_hash_overwrite_with_free_cb(void **state);
+void test_sss_ptr_hash_with_lookup_cb(void **state);
+void test_sss_ptr_hash_without_cb(void **state);
+
 
 #endif /* __TESTS__CMOCKA__TEST_UTILS_H__ */
