@@ -178,7 +178,7 @@ int main(int argc, const char *argv[])
         cmocka_unit_test(test_run_as_root_daemon),
     };
 
-    /* Set debug level to invalid value so we can deside if -d 0 was used. */
+    /* Set debug level to invalid value so we can decide if -d 0 was used. */
     debug_level = SSSDBG_INVALID;
 
     pc = poptGetContext(argv[0], argc, argv, long_options, 0);
@@ -196,13 +196,13 @@ int main(int argc, const char *argv[])
     DEBUG_CLI_INIT(debug_level);
 
     /* Even though normally the tests should clean up after themselves
-     * they might not after a failed run. Remove the old db to be sure */
+     * they might not after a failed run. Remove the old DB to be sure */
     tests_set_cwd();
     test_dom_suite_cleanup(TEST_DB_PATH, CONFDB_FILE, NULL);
     test_dom_suite_setup(TEST_DB_PATH);
 
     rv = cmocka_run_group_tests(tests, NULL, NULL);
-    if (rv != 0) {
+    if (rv == 0) {
         test_dom_suite_cleanup(TEST_DB_PATH, CONFDB_FILE, NULL);
     }
 
