@@ -48,16 +48,15 @@ START_TEST(test_debug_convert_old_level_old_format)
         SSSDBG_TRACE_FUNC,
         SSSDBG_TRACE_LIBS,
         SSSDBG_TRACE_INTERNAL,
-        SSSDBG_TRACE_ALL | SSSDBG_BE_FO
+        SSSDBG_TRACE_ALL | SSSDBG_BE_FO,
+        SSSDBG_TRACE_LDB
     };
 
-    for (old_level = 0; old_level <= 9; old_level++) {
+    for (old_level = 0; old_level < N_ELEMENTS(levels); old_level++) {
         expected_level |= levels[old_level];
 
-        char *msg = NULL;
-        msg = talloc_asprintf(NULL, "Invalid conversion of %d", old_level);
-        fail_unless(debug_convert_old_level(old_level) == expected_level, msg);
-        talloc_free(msg);
+        fail_unless(debug_convert_old_level(old_level) == expected_level,
+                    "Invalid conversion of %d", old_level);
     }
 }
 END_TEST
@@ -108,6 +107,10 @@ START_TEST(test_debug_convert_old_level_new_format)
         debug_convert_old_level(SSSDBG_TRACE_ALL) == SSSDBG_TRACE_ALL,
         "Invalid conversion of SSSDBG_TRACE_ALL"
     );
+    fail_unless(
+        debug_convert_old_level(SSSDBG_TRACE_LDB) == SSSDBG_TRACE_LDB,
+        "Invalid conversion of SSSDBG_TRACE_LDB"
+    );
     fail_unless(
         debug_convert_old_level(SSSDBG_MASK_ALL) == SSSDBG_MASK_ALL,
         "Invalid conversion of SSSDBG_MASK_ALL"
@@ -190,29 +193,29 @@ int test_helper_debug_check_message(int level)
     msg[fsize] = '\0';
 
     if (debug_timestamps == 1) {
-        char time_day[4] = {'\0', '\0', '\0', '\0'};
-        char time_month[4] = {'\0', '\0', '\0', '\0'};
-        int time_day_num = 0;
         int time_hour = 0;
         int time_min = 0;
         int time_sec = 0;
         int time_usec = 0;
+        int time_day = 0;
+        int time_month = 0;
         int time_year = 0;
         int scan_return = 0;
 
         if (debug_microseconds == 0) {
-            scan_return = sscanf(msg, "(%s %s %d %d:%d:%d %d)", time_day, time_month,
-                                 &time_day_num, &time_hour, &time_min, &time_sec, &time_year);
+            scan_return = sscanf(msg, "(%d-%d-%d %d:%d:%d)", &time_year,
+                                 &time_month, &time_day, &time_hour,
+                                 &time_min, &time_sec);
 
-            if (scan_return != 7) {
+            if (scan_return != 6) {
                 ret = DEBUG_TEST_NOK_TS;
                 goto done;
             }
             compare_to = talloc_asprintf(ctx,
-                                         "(%s %s %2d %.2d:%.2d:%.2d %.4d) "
+                                         "(%d-%02d-%02d %2d:%02d:%02d): "
                                          "[%s] [%s] (%#.4x): %s\n",
-                                         time_day, time_month, time_day_num,
-                                         time_hour, time_min, time_sec, time_year,
+                                         time_year, time_month, time_day,
+                                         time_hour, time_min, time_sec,
                                          debug_prg_name, function, level, body);
             if (compare_to == NULL) {
                 _errno = ENOMEM;
@@ -220,20 +223,20 @@ int test_helper_debug_check_message(int level)
                 goto done;
             }
         } else {
-            scan_return = sscanf(msg, "(%s %s %d %d:%d:%d:%d %d)", time_day, time_month,
-                                 &time_day_num, &time_hour, &time_min, &time_sec,
-                                 &time_usec, &time_year);
+            scan_return = sscanf(msg, "(%d-%d-%d %d:%d:%d:%d)", &time_year,
+                                 &time_month, &time_day, &time_hour,
+                                 &time_min, &time_sec, &time_usec);
 
-            if (scan_return != 8) {
+            if (scan_return != 7) {
                 ret = DEBUG_TEST_NOK_TS;
                 goto done;
             }
             compare_to = talloc_asprintf(ctx,
-                                         "(%s %s %2d %.2d:%.2d:%.2d:%.6d %.4d) "
+                                         "(%d-%02d-%02d %2d:%02d:%02d:%.6d): "
                                          "[%s] [%s] (%#.4x): %s\n",
-                                         time_day, time_month, time_day_num,
+                                         time_year, time_month, time_day,
                                          time_hour, time_min, time_sec, time_usec,
-                                         time_year, debug_prg_name, function, level, body);
+                                         debug_prg_name, function, level, body);
             if (compare_to == NULL) {
                 _errno = ENOMEM;
                 ret = DEBUG_TEST_ERROR;
@@ -335,9 +338,9 @@ START_TEST(test_debug_is_set_single_no_timestamp)
         SSSDBG_TRACE_FUNC,
         SSSDBG_TRACE_LIBS,
         SSSDBG_TRACE_INTERNAL,
-        SSSDBG_TRACE_ALL
+        SSSDBG_TRACE_ALL,
+        SSSDBG_TRACE_LDB
     };
-    char *error_msg;
 
     debug_timestamps = 0;
     debug_microseconds = 0;
@@ -345,21 +348,19 @@ START_TEST(test_debug_is_set_single_no_timestamp)
     debug_prg_name = "sssd";
     sss_set_logger(sss_logger_str[FILES_LOGGER]);
 
-    for (i = 0; i <= 9; i++) {
+    for (i = 0; i < N_ELEMENTS(levels); i++) {
         debug_level = levels[i];
 
         errno = 0;
         result = test_helper_debug_check_message(levels[i]);
 
-        if (result == DEBUG_TEST_ERROR) {
-            error_msg = strerror(errno);
-            fail(error_msg);
-        }
+        fail_if(result == DEBUG_TEST_ERROR,
+                "Expecting DEBUG_TEST_ERROR, got: %d, error: %s",
+                result, strerror(errno));
 
-        char *msg = NULL;
-        msg = talloc_asprintf(NULL, "Test of level %#.4x failed - message don't match", levels[i]);
-        fail_unless(result == EOK, msg);
-        talloc_free(msg);
+        fail_unless(result == EOK,
+                    "Test of level %#.4x failed - message don't match",
+                    levels[i]);
     }
 }
 END_TEST
@@ -378,9 +379,9 @@ START_TEST(test_debug_is_set_single_timestamp)
         SSSDBG_TRACE_FUNC,
         SSSDBG_TRACE_LIBS,
         SSSDBG_TRACE_INTERNAL,
-        SSSDBG_TRACE_ALL
+        SSSDBG_TRACE_ALL,
+        SSSDBG_TRACE_LDB
     };
-    char *error_msg;
 
     debug_timestamps = 1;
     debug_microseconds = 0;
@@ -389,26 +390,22 @@ START_TEST(test_debug_is_set_single_timestamp)
     sss_set_logger(sss_logger_str[FILES_LOGGER]);
 
 
-    for (i = 0; i <= 9; i++) {
+    for (i = 0; i < N_ELEMENTS(levels); i++) {
         debug_level = levels[i];
 
         errno = 0;
         result = test_helper_debug_check_message(levels[i]);
 
-        if (result == DEBUG_TEST_ERROR) {
-            error_msg = strerror(errno);
-            fail(error_msg);
-        }
+        fail_if(result == DEBUG_TEST_ERROR,
+                "Expecting DEBUG_TEST_ERROR, got: %d, error: %s",
+                result, strerror(errno));
 
-        char *msg = NULL;
+        fail_if(result == DEBUG_TEST_NOK_TS,
+                "Test of level %#.4x failed - invalid timestamp", levels[i]);
 
-        msg = talloc_asprintf(NULL, "Test of level %#.4x failed - invalid timestamp", levels[i]);
-        fail_if(result == DEBUG_TEST_NOK_TS, msg);
-        talloc_free(msg);
-
-        msg = talloc_asprintf(NULL, "Test of level %#.4x failed - message don't match", levels[i]);
-        fail_unless(result == EOK, msg);
-        talloc_free(msg);
+        fail_unless(result == EOK,
+                    "Test of level %#.4x failed - message don't match",
+                    levels[i]);
     }
 }
 END_TEST
@@ -427,9 +424,9 @@ START_TEST(test_debug_is_set_single_timestamp_microseconds)
         SSSDBG_TRACE_FUNC,
         SSSDBG_TRACE_LIBS,
         SSSDBG_TRACE_INTERNAL,
-        SSSDBG_TRACE_ALL
+        SSSDBG_TRACE_ALL,
+        SSSDBG_TRACE_LDB
     };
-    char *error_msg;
 
     debug_timestamps = 1;
     debug_microseconds = 1;
@@ -438,26 +435,22 @@ START_TEST(test_debug_is_set_single_timestamp_microseconds)
     sss_set_logger(sss_logger_str[FILES_LOGGER]);
 
 
-    for (i = 0; i <= 9; i++) {
+    for (i = 0; i < N_ELEMENTS(levels); i++) {
         debug_level = levels[i];
 
         errno = 0;
         result = test_helper_debug_check_message(levels[i]);
 
-        if (result == DEBUG_TEST_ERROR) {
-            error_msg = strerror(errno);
-            fail(error_msg);
-        }
+        fail_if(result == DEBUG_TEST_ERROR,
+                "Expecting DEBUG_TEST_ERROR, got: %d, error: %s",
+                result, strerror(errno));
 
-        char *msg = NULL;
+        fail_if(result == DEBUG_TEST_NOK_TS,
+                "Test of level %#.4x failed - invalid timestamp", levels[i]);
 
-        msg = talloc_asprintf(NULL, "Test of level %#.4x failed - invalid timestamp", levels[i]);
-        fail_if(result == DEBUG_TEST_NOK_TS, msg);
-        talloc_free(msg);
-
-        msg = talloc_asprintf(NULL, "Test of level %#.4x failed - message don't match", levels[i]);
-        fail_unless(result == EOK, msg);
-        talloc_free(msg);
+        fail_unless(result == EOK,
+                    "Test of level %#.4x failed - message don't match",
+                    levels[i]);
     }
 }
 END_TEST
@@ -477,9 +470,9 @@ START_TEST(test_debug_is_notset_no_timestamp)
         SSSDBG_TRACE_FUNC,
         SSSDBG_TRACE_LIBS,
         SSSDBG_TRACE_INTERNAL,
-        SSSDBG_TRACE_ALL
+        SSSDBG_TRACE_ALL,
+        SSSDBG_TRACE_LDB
     };
-    char *error_msg;
 
     debug_timestamps = 0;
     debug_microseconds = 0;
@@ -488,23 +481,19 @@ START_TEST(test_debug_is_notset_no_timestamp)
     sss_set_logger(sss_logger_str[FILES_LOGGER]);
 
 
-    for (i = 0; i <= 9; i++) {
+    for (i = 0; i < N_ELEMENTS(levels); i++) {
         debug_level = all_set & ~levels[i];
 
         errno = 0;
         result = test_helper_debug_is_empty_message(levels[i]);
 
-        if (result == DEBUG_TEST_ERROR) {
-            error_msg = strerror(errno);
-            fail(error_msg);
-        }
+        fail_if(result == DEBUG_TEST_ERROR,
+                "Expecting DEBUG_TEST_ERROR, got: %d, error: %s",
+                result, strerror(errno));
 
-        char *msg = NULL;
-        msg = talloc_asprintf(NULL,
-                              "Test of level %#.4x failed - message has been written",
-                              levels[i]);
-        fail_unless(result == EOK, msg);
-        talloc_free(msg);
+        fail_unless(result == EOK,
+                    "Test of level %#.4x failed - message has been written",
+                    levels[i]);
     }
 }
 END_TEST
@@ -524,9 +513,9 @@ START_TEST(test_debug_is_notset_timestamp)
         SSSDBG_TRACE_FUNC,
         SSSDBG_TRACE_LIBS,
         SSSDBG_TRACE_INTERNAL,
-        SSSDBG_TRACE_ALL
+        SSSDBG_TRACE_ALL,
+        SSSDBG_TRACE_LDB
     };
-    char *error_msg;
 
     debug_timestamps = 0;
     debug_microseconds = 0;
@@ -535,23 +524,19 @@ START_TEST(test_debug_is_notset_timestamp)
     sss_set_logger(sss_logger_str[FILES_LOGGER]);
 
 
-    for (i = 0; i <= 9; i++) {
+    for (i = 0; i < N_ELEMENTS(levels); i++) {
         debug_level = all_set & ~levels[i];
 
         errno = 0;
         result = test_helper_debug_is_empty_message(levels[i]);
 
-        if (result == DEBUG_TEST_ERROR) {
-            error_msg = strerror(errno);
-            fail(error_msg);
-        }
+        fail_if(result == DEBUG_TEST_ERROR,
+                "Expecting DEBUG_TEST_ERROR, got: %d, error: %s",
+                result, strerror(errno));
 
-        char *msg = NULL;
-        msg = talloc_asprintf(NULL,
-                        "Test of level %#.4x failed - message has been written",
-                        levels[i]);
-        fail_unless(result == EOK, msg);
-        talloc_free(msg);
+        fail_unless(result == EOK,
+                    "Test of level %#.4x failed - message has been written",
+                    levels[i]);
     }
 }
 END_TEST
@@ -571,9 +556,9 @@ START_TEST(test_debug_is_notset_timestamp_microseconds)
         SSSDBG_TRACE_FUNC,
         SSSDBG_TRACE_LIBS,
         SSSDBG_TRACE_INTERNAL,
-        SSSDBG_TRACE_ALL
+        SSSDBG_TRACE_ALL,
+        SSSDBG_TRACE_LDB
     };
-    char *error_msg;
 
     debug_timestamps = 0;
     debug_microseconds = 1;
@@ -581,23 +566,19 @@ START_TEST(test_debug_is_notset_timestamp_microseconds)
     debug_prg_name = "sssd";
     sss_set_logger(sss_logger_str[FILES_LOGGER]);
 
-    for (i = 0; i <= 9; i++) {
+    for (i = 0; i < N_ELEMENTS(levels); i++) {
         debug_level = all_set & ~levels[i];
 
         errno = 0;
         result = test_helper_debug_is_empty_message(levels[i]);
 
-        if (result == DEBUG_TEST_ERROR) {
-            error_msg = strerror(errno);
-            fail(error_msg);
-        }
+        fail_if(result == DEBUG_TEST_ERROR,
+                "Expecting DEBUG_TEST_ERROR, got: %d, error: %s",
+                result, strerror(errno));
 
-        char *msg = NULL;
-        msg = talloc_asprintf(NULL,
-                        "Test of level %#.4x failed - message has been written",
-                        levels[i]);
-        fail_unless(result == EOK, msg);
-        talloc_free(msg);
+        fail_unless(result == EOK,
+                    "Test of level %#.4x failed - message has been written",
+                    levels[i]);
     }
 }
 END_TEST
@@ -616,17 +597,17 @@ START_TEST(test_debug_is_set_true)
         SSSDBG_TRACE_FUNC,
         SSSDBG_TRACE_LIBS,
         SSSDBG_TRACE_INTERNAL,
-        SSSDBG_TRACE_ALL
+        SSSDBG_TRACE_ALL,
+        SSSDBG_TRACE_LDB
     };
 
     debug_level = SSSDBG_MASK_ALL;
 
-    for (i = 0; i <= 9; i++) {
+    for (i = 0; i < N_ELEMENTS(levels); i++) {
         result = DEBUG_IS_SET(levels[i]);
-        char *msg = NULL;
-        msg = talloc_asprintf(NULL, "Test of level %#.4x failed - result is 0x%.4x", levels[i], result);
-        fail_unless(result > 0, msg);
-        talloc_free(msg);
+        fail_unless(result > 0,
+                    "Test of level %#.4x failed - result is 0x%.4x",
+                    levels[i], result);
     }
 }
 END_TEST
@@ -646,17 +627,17 @@ START_TEST(test_debug_is_set_false)
         SSSDBG_TRACE_FUNC,
         SSSDBG_TRACE_LIBS,
         SSSDBG_TRACE_INTERNAL,
-        SSSDBG_TRACE_ALL
+        SSSDBG_TRACE_ALL,
+        SSSDBG_TRACE_LDB
     };
 
-    for (i = 0; i <= 9; i++) {
+    for (i = 0; i < N_ELEMENTS(levels); i++) {
         debug_level = all_set & ~levels[i];
 
         result = DEBUG_IS_SET(levels[i]);
-        char *msg = NULL;
-        msg = talloc_asprintf(NULL, "Test of level %#.4x failed - result is 0x%.4x", levels[i], result);
-        fail_unless(result == 0, msg);
-        talloc_free(msg);
+        fail_unless(result == 0,
+                    "Test of level %#.4x failed - result is 0x%.4x",
+                    levels[i], result);
     }
 }
 END_TEST
