@@ -303,6 +303,25 @@ def sanity_rfc2307_bis(request, ldap_conn):
     return None
 
 
+@pytest.fixture
+def member_with_different_cases_rfc2307_bis(request, ldap_conn):
+    """
+    Create a group where the user DN values of the RFC2307bis member attribute
+    differ in case from the original DN of the user object.
+    """
+    ent_list = ldap_ent.List(ldap_conn.ds_inst.base_dn)
+    ent_list.add_user("user1", 1001, 2001)
+    ent_list.add_user("user2", 1002, 2002)
+
+    ent_list.add_group_bis("two_user_group", 2012, ["USER1", "uSeR2"])
+
+    create_ldap_fixture(request, ldap_conn, ent_list)
+    conf = format_basic_conf(ldap_conn, SCHEMA_RFC2307_BIS)
+    create_conf_fixture(request, conf)
+    create_sssd_fixture(request)
+    return None
+
+
 def expected_list_to_name_dict(entries):
     return dict((u["name"], u) for u in entries)
 
@@ -394,6 +413,22 @@ def test_sanity_rfc2307_bis(ldap_conn, sanity_rfc2307_bis):
         grp.getgrgid(1)
 
 
+def test_member_with_different_cases_rfc2307_bis(
+                                      ldap_conn,
+                                      member_with_different_cases_rfc2307_bis):
+    """
+    Regression test for https://bugzilla.redhat.com/show_bug.cgi?id=1817122
+    Make sure that group members are added properly to the group even if the
+    user DN in the RFC2307bis member attribute differs in case from the
+    original DN of the user object.
+    """
+    group_pattern = expected_list_to_name_dict([
+        dict(name='two_user_group', passwd='*', gid=2012,
+             mem=ent.contains_only("user1", "user2")),
+    ])
+    ent.assert_each_group_by_name(group_pattern)
+
+
 @pytest.fixture
 def refresh_after_cleanup_task(request, ldap_conn):
     ent_list = ldap_ent.List(ldap_conn.ds_inst.base_dn)
@@ -1155,6 +1190,18 @@ def test_nss_filters(ldap_conn, sanity_nss_filter):
     with pytest.raises(KeyError):
         grp.getgrgid(14)
 
+    # test initgroups - user1 is member of group_two_one_user_groups (2019)
+    # which is filtered out
+    (res, errno, gids) = sssd_id.call_sssd_initgroups("user1", 2001)
+    assert res == sssd_id.NssReturnCode.SUCCESS
+
+    user_with_group_ids = [2001, 2012, 2015, 2017, 2018]
+    assert sorted(gids) == sorted(user_with_group_ids), \
+        "result: %s\n expected %s" % (
+            ", ".join(["%s" % s for s in sorted(gids)]),
+            ", ".join(["%s" % s for s in sorted(user_with_group_ids)])
+        )
+
 
 @pytest.fixture
 def sanity_nss_filter_cached(request, ldap_conn):
@@ -1261,7 +1308,7 @@ def test_ldap_auto_private_groups_direct(ldap_conn, mpg_setup):
     """
     Integration test for auto_private_groups
 
-    See also ticket https://pagure.io/SSSD/sssd/issue/1872
+    See also ticket https://github.com/SSSD/sssd/issues/2914
     """
     # Make sure the user's GID is taken from their uidNumber
     ent.assert_passwd_by_name("user1", dict(name="user1", uid=1001, gid=1001))
@@ -1416,7 +1463,7 @@ def test_ldap_auto_private_groups_direct_no_gid(ldap_conn, mpg_setup_no_gid):
     no GID assigned at all can be resolved including their autogenerated
     primary group.
 
-    See also ticket https://pagure.io/SSSD/sssd/issue/1872
+    See also ticket https://github.com/SSSD/sssd/issues/2914
     """
     # Make sure the user's GID is taken from their uidNumber
     ent.assert_passwd_by_name("user1", dict(name="user1", uid=1001, gid=1001))
@@ -1516,7 +1563,7 @@ def test_ldap_auto_private_groups_hybrid_direct(ldap_conn, mpg_setup_hybrid):
     Integration test for auto_private_groups=hybrid. This test checks the
     resolution of the users and their groups.
 
-    See also ticket https://pagure.io/SSSD/sssd/issue/1872
+    See also ticket https://github.com/SSSD/sssd/issues/2914
     """
     # Make sure the user's GID is taken from their gidNumber, if available
     ent.assert_passwd_by_name("user_with_group",
@@ -1590,7 +1637,7 @@ def test_ldap_auto_private_groups_hybrid_priv_group_byname(ldap_conn,
     Integration test for auto_private_groups=hybrid. This test checks the
     resolution of user private groups by name.
 
-    See also ticket https://pagure.io/SSSD/sssd/issue/1872
+    See also ticket https://github.com/SSSD/sssd/issues/2914
     """
     # gidNumber is resolvable by name..
     ent.assert_group_by_name("user_with_group_pvt",
@@ -1631,7 +1678,7 @@ def test_ldap_auto_private_groups_hybrid_priv_group_byid(ldap_conn,
     Integration test for auto_private_groups=hybrid. This test checks the
     resolution of user private groups by name.
 
-    See also ticket https://pagure.io/SSSD/sssd/issue/1872
+    See also ticket https://github.com/SSSD/sssd/issues/2914
     """
     # Make sure the private group of user who has this group set in their
     # gidNumber is resolvable by ID
@@ -1667,7 +1714,7 @@ def test_ldap_auto_private_groups_hybrid_priv_group_byid(ldap_conn,
 def test_ldap_auto_private_groups_hybrid_name_gid_identical(ldap_conn,
                                                             mpg_setup_hybrid):
     """
-    See also ticket https://pagure.io/SSSD/sssd/issue/1872
+    See also ticket https://github.com/SSSD/sssd/issues/2914
     """
     ent.assert_passwd_by_name("u_g_same_name",
                               dict(name="u_g_same_name",
@@ -1689,7 +1736,7 @@ def test_ldap_auto_private_groups_hybrid_name_gid_identical(ldap_conn,
 
 def test_ldap_auto_private_groups_hybrid_initgr(ldap_conn, mpg_setup_hybrid):
     """
-    See also ticket https://pagure.io/SSSD/sssd/issue/1872
+    See also ticket https://github.com/SSSD/sssd/issues/2914
     """
     user_without_group_ids = [1004, 10040, 10041]
     (res, errno, gids) = sssd_id.call_sssd_initgroups("user_with_real_group",
@@ -1770,7 +1817,7 @@ def test_rename_incomplete_group_same_dn(ldap_conn, rename_setup_with_name):
     attribute "name" that is purposefully different from the CN and make
     sure the group names are reflected in name
 
-    Regression test for https://pagure.io/SSSD/sssd/issue/3282
+    Regression test for https://github.com/SSSD/sssd/issues/4315
     """
     pvt_dn1 = 'cn=user1_private,ou=Groups,' + ldap_conn.ds_inst.base_dn
     pvt_dn2 = 'cn=user2_private,ou=Groups,' + ldap_conn.ds_inst.base_dn
@@ -1826,7 +1873,7 @@ def test_rename_incomplete_group_rdn_changed(ldap_conn, rename_setup_cleanup):
     Please note that with many directories (AD, IPA), the code can rely on
     other heuristics (SID, UUID) to find out the group is in fact the same.
 
-    Regression test for https://pagure.io/SSSD/sssd/issue/3282
+    Regression test for https://github.com/SSSD/sssd/issues/4315
     """
     pvt_dn = 'cn=user1_private,ou=Groups,' + ldap_conn.ds_inst.base_dn
     group1_dn = 'cn=group1,ou=Groups,' + ldap_conn.ds_inst.base_dn
@@ -2031,7 +2078,7 @@ def test_conflicting_mail_addresses_and_fqdn(ldap_conn,
     Test that we handle the case where one user's mail address is the
     same as another user's FQDN
 
-    This is a regression test for https://pagure.io/SSSD/sssd/issue/3607
+    This is a regression test for https://github.com/SSSD/sssd/issues/4630
     """
     # With #3607 unfixed, these two lookups would prime the cache with
     # nameAlias: emailuser@LDAP for both entries..
