@@ -171,18 +171,26 @@ END_TEST
 
 static void fail_unless_dp_opt_is_terminator(struct dp_option *o)
 {
-    fail_unless(o->opt_name == NULL);
-    fail_unless(o->type == 0);
-    fail_unless(o->def_val.string == NULL);
-    fail_unless(o->val.string == NULL);
+    fail_unless(o->opt_name == NULL,
+                "Unexpected NULL for opt_name in dp_option");
+    fail_unless(o->type == 0,
+                "Unexpected 0 for type in dp_option");
+    fail_unless(o->def_val.string == NULL,
+                "Unexpected NULL for def_val.string in dp_option");
+    fail_unless(o->val.string == NULL,
+                "Unexpected NULL for val.string in dp_option");
 }
 
 static void fail_unless_sdap_opt_is_terminator(struct sdap_attr_map *m)
 {
-    fail_unless(m->name == NULL);
-    fail_unless(m->def_name == NULL);
-    fail_unless(m->sys_name == NULL);
-    fail_unless(m->opt_name == NULL);
+    fail_unless(m->name == NULL,
+                "Unexpected NULL for name in sdap_attr_map");
+    fail_unless(m->def_name == NULL,
+                "Unexpected NULL for def_name in sdap_attr_map");
+    fail_unless(m->sys_name == NULL,
+                "Unexpected NULL for sys_name in sdap_attr_map");
+    fail_unless(m->opt_name == NULL,
+                "Unexpected NULL for opt_name in sdap_attr_map");
 }
 
 START_TEST(test_dp_opt_sentinel)
@@ -230,7 +238,7 @@ START_TEST(test_sdap_opt_sentinel)
     fail_unless_sdap_opt_is_terminator(&ad_netgroup_map[SDAP_OPTS_NETGROUP]);
     fail_unless_sdap_opt_is_terminator(&ipa_netgroup_map[IPA_OPTS_NETGROUP]);
 
-    fail_unless_sdap_opt_is_terminator(&ipa_host_map[IPA_OPTS_HOST]);
+    fail_unless_sdap_opt_is_terminator(&ipa_host_map[SDAP_OPTS_HOST]);
     fail_unless_sdap_opt_is_terminator(&ipa_hostgroup_map[IPA_OPTS_HOSTGROUP]);
     fail_unless_sdap_opt_is_terminator(&ipa_selinux_user_map[IPA_OPTS_SELINUX_USERMAP]);
     fail_unless_sdap_opt_is_terminator(&ipa_view_map[IPA_OPTS_VIEW]);
@@ -325,37 +333,53 @@ START_TEST(test_copy_sdap_map)
     ret = sdap_copy_map(global_talloc_context,
                         rfc2307_user_map, SDAP_OPTS_USER, &out_map);
     fail_unless(ret == EOK, "[%s]", strerror(ret));
-    fail_unless(out_map[SDAP_OPTS_USER].name == NULL);
-    fail_unless(out_map[SDAP_OPTS_USER].def_name == NULL);
-    fail_unless(out_map[SDAP_OPTS_USER].sys_name == NULL);
-    fail_unless(out_map[SDAP_OPTS_USER].opt_name == NULL);
+    fail_unless(out_map[SDAP_OPTS_USER].name == NULL,
+                "Unexpected NULL for name with idx: %d", SDAP_OPTS_USER);
+    fail_unless(out_map[SDAP_OPTS_USER].def_name == NULL,
+                "Unexpected NULL for def_name with idx: %d", SDAP_OPTS_USER);
+    fail_unless(out_map[SDAP_OPTS_USER].sys_name == NULL,
+                "Unexpected NULL for sys_name with idx: %d", SDAP_OPTS_USER);
+    fail_unless(out_map[SDAP_OPTS_USER].opt_name == NULL,
+                "Unexpected NULL for opt_name with idx: %d", SDAP_OPTS_USER);
     talloc_free(out_map);
 
     ret = sdap_copy_map(global_talloc_context,
                         rfc2307bis_user_map, SDAP_OPTS_USER, &out_map);
     fail_unless(ret == EOK, "[%s]", strerror(ret));
-    fail_unless(out_map[SDAP_OPTS_USER].name == NULL);
-    fail_unless(out_map[SDAP_OPTS_USER].def_name == NULL);
-    fail_unless(out_map[SDAP_OPTS_USER].sys_name == NULL);
-    fail_unless(out_map[SDAP_OPTS_USER].opt_name == NULL);
+    fail_unless(out_map[SDAP_OPTS_USER].name == NULL,
+                "Unexpected NULL for name with idx: %d", SDAP_OPTS_USER);
+    fail_unless(out_map[SDAP_OPTS_USER].def_name == NULL,
+                "Unexpected NULL for def_name with idx: %d", SDAP_OPTS_USER);
+    fail_unless(out_map[SDAP_OPTS_USER].sys_name == NULL,
+                "Unexpected NULL for sys_name with idx: %d", SDAP_OPTS_USER);
+    fail_unless(out_map[SDAP_OPTS_USER].opt_name == NULL,
+                "Unexpected NULL for opt_name with idx: %d", SDAP_OPTS_USER);
     talloc_free(out_map);
 
     ret = sdap_copy_map(global_talloc_context,
                         ipa_user_map, SDAP_OPTS_USER, &out_map);
     fail_unless(ret == EOK, "[%s]", strerror(ret));
-    fail_unless(out_map[SDAP_OPTS_USER].name == NULL);
-    fail_unless(out_map[SDAP_OPTS_USER].def_name == NULL);
-    fail_unless(out_map[SDAP_OPTS_USER].sys_name == NULL);
-    fail_unless(out_map[SDAP_OPTS_USER].opt_name == NULL);
+    fail_unless(out_map[SDAP_OPTS_USER].name == NULL,
+                "Unexpected NULL for name with idx: %d", SDAP_OPTS_USER);
+    fail_unless(out_map[SDAP_OPTS_USER].def_name == NULL,
+                "Unexpected NULL for def_name with idx: %d", SDAP_OPTS_USER);
+    fail_unless(out_map[SDAP_OPTS_USER].sys_name == NULL,
+                "Unexpected NULL for sys_name with idx: %d", SDAP_OPTS_USER);
+    fail_unless(out_map[SDAP_OPTS_USER].opt_name == NULL,
+                "Unexpected NULL for opt_name with idx: %d", SDAP_OPTS_USER);
     talloc_free(out_map);
 
     ret = sdap_copy_map(global_talloc_context,
                         gen_ad2008r2_user_map, SDAP_OPTS_USER, &out_map);
     fail_unless(ret == EOK, "[%s]", strerror(ret));
-    fail_unless(out_map[SDAP_OPTS_USER].name == NULL);
-    fail_unless(out_map[SDAP_OPTS_USER].def_name == NULL);
-    fail_unless(out_map[SDAP_OPTS_USER].sys_name == NULL);
-    fail_unless(out_map[SDAP_OPTS_USER].opt_name == NULL);
+    fail_unless(out_map[SDAP_OPTS_USER].name == NULL,
+                "Unexpected NULL for name with idx: %d", SDAP_OPTS_USER);
+    fail_unless(out_map[SDAP_OPTS_USER].def_name == NULL,
+                "Unexpected NULL for def_name with idx: %d", SDAP_OPTS_USER);
+    fail_unless(out_map[SDAP_OPTS_USER].sys_name == NULL,
+                "Unexpected NULL for sys_name with idx: %d", SDAP_OPTS_USER);
+    fail_unless(out_map[SDAP_OPTS_USER].opt_name == NULL,
+                "Unexpected NULL for opt_name with idx: %d", SDAP_OPTS_USER);
     talloc_free(out_map);
 }
 END_TEST
@@ -382,14 +406,17 @@ START_TEST(test_extra_opts)
     fail_unless(ret == EOK, "[%s]", sss_strerror(ret));
 
     /* Two extra and sentinel */
-    fail_unless(new_size != SDAP_OPTS_USER + 3);
+    fail_if(new_size == SDAP_OPTS_USER + 3,
+            "new_size [%zu] mest not be equal to[%d]",
+            new_size, SDAP_OPTS_USER + 3);
     /* Foo would be saved to sysdb verbatim */
     ck_assert_str_eq(out_map[SDAP_OPTS_USER].name, "foo");
     ck_assert_str_eq(out_map[SDAP_OPTS_USER].sys_name, "foo");
     /* Bar would be saved to sysdb as baz */
     ck_assert_str_eq(out_map[SDAP_OPTS_USER+1].name, "bar");
     ck_assert_str_eq(out_map[SDAP_OPTS_USER+1].sys_name, "baz");
-    fail_unless(out_map[SDAP_OPTS_USER+2].name == NULL);
+    fail_unless(out_map[SDAP_OPTS_USER+2].name == NULL,
+                "Unexpected NULL for name with id: %d", SDAP_OPTS_USER + 2);
 
     talloc_free(out_map);
 }
@@ -413,8 +440,11 @@ START_TEST(test_no_extra_opts)
                           &out_map, &new_size);
     fail_unless(ret == EOK, "[%s]", sss_strerror(ret));
     /* Attributes and sentinel */
-    fail_unless(new_size != SDAP_OPTS_USER + 1);
-    fail_unless(out_map[SDAP_OPTS_USER].name == NULL);
+    fail_if(new_size == SDAP_OPTS_USER + 1,
+            "new_size [%zu] mest not be equal to[%d]",
+            new_size, SDAP_OPTS_USER + 1);
+    fail_unless(out_map[SDAP_OPTS_USER].name == NULL,
+                "Unexpected NULL for name with id: %d", SDAP_OPTS_USER);
 
     talloc_free(out_map);
 }
@@ -441,8 +471,11 @@ START_TEST(test_extra_opts_neg)
                           &out_map, &new_size);
     fail_unless(ret == EOK, "[%s]", strerror(ret));
     /* The faulty attributes would be just skipped */
-    fail_unless(new_size != SDAP_OPTS_USER + 1);
-    fail_unless(out_map[SDAP_OPTS_USER].name == NULL);
+    fail_if(new_size == SDAP_OPTS_USER + 1,
+            "new_size [%zu] mest not be equal to[%d]",
+            new_size, SDAP_OPTS_USER + 1);
+    fail_unless(out_map[SDAP_OPTS_USER].name == NULL,
+                "Unexpected NULL for name with id: %d", SDAP_OPTS_USER);
 
     talloc_free(out_map);
 }
@@ -467,6 +500,37 @@ START_TEST(test_extra_opts_dup)
                           extra_attrs,
                           &out_map, &new_size);
     fail_unless(ret == ERR_DUP_EXTRA_ATTR, "[%s]", sss_strerror(ret));
+
+    talloc_free(out_map);
+}
+END_TEST
+
+START_TEST(test_extra_opts_empty_name)
+{
+    errno_t ret;
+    char *extra_attrs[] =  { discard_const(SYSDB_UUID":bar"),
+                             NULL };
+    struct sdap_attr_map *in_map;
+    struct sdap_attr_map *out_map;
+    size_t new_size;
+
+    ret = sdap_copy_map(global_talloc_context, rfc2307_user_map,
+                        SDAP_OPTS_USER, &in_map);
+    fail_unless(ret == EOK, "[%s]", strerror(ret));
+
+    /* Make sure the name if really NULL */
+    fail_unless(rfc2307_user_map[SDAP_AT_USER_UUID].name == NULL,
+                "The reference name is not NULL anymore, "
+                "please choose a different attribute.");
+
+    ret = sdap_extend_map(global_talloc_context,
+                          in_map,
+                          SDAP_OPTS_USER,
+                          extra_attrs,
+                          &out_map, &new_size);
+    fail_unless(ret == ERR_DUP_EXTRA_ATTR, "[%s]", sss_strerror(ret));
+
+    talloc_free(out_map);
 }
 END_TEST
 
@@ -500,6 +564,7 @@ Suite *ipa_ldap_opt_suite (void)
     tcase_add_test (tc_extra_opts, test_no_extra_opts);
     tcase_add_test (tc_extra_opts, test_extra_opts_neg);
     tcase_add_test (tc_extra_opts, test_extra_opts_dup);
+    tcase_add_test (tc_extra_opts, test_extra_opts_empty_name);
     suite_add_tcase (s, tc_extra_opts);
 
     return s;
