@@ -75,7 +75,7 @@ START_TEST(resp_str_to_array_test)
                                                                 strerror(ret));
         if (ret == 0) {
             fail_unless(uid_count == s2a_data[c].exp_count,
-                        "Wrong number of values, expected [%d], got [%d].",
+                        "Wrong number of values, expected [%zu], got [%zu].",
                         s2a_data[c].exp_count, uid_count);
             for (d = 0; d < s2a_data[c].exp_count; d++) {
                 fail_unless(uids[d] == s2a_data[c].exp_uids[d],
@@ -148,7 +148,7 @@ int main(int argc, const char *argv[])
         POPT_TABLEEND
     };
 
-    /* Set debug level to invalid value so we can deside if -d 0 was used. */
+    /* Set debug level to invalid value so we can decide if -d 0 was used. */
     debug_level = SSSDBG_INVALID;
 
     pc = poptGetContext(argv[0], argc, argv, long_options, 0);
