@@ -101,7 +101,7 @@ int main(int argc, const char **argv)
         BAD_POPT_PARAMS(pc, _("Specify group to modify\n"), ret, fini);
     }
 
-    DEBUG_INIT(pc_debug);
+    DEBUG_CLI_INIT(pc_debug);
 
     CHECK_ROOT(ret, debug_prg_name);
 
@@ -125,7 +125,7 @@ int main(int argc, const char **argv)
         goto fini;
     }
     /* check the username to be able to give sensible error message */
-    ret = sysdb_getgrnam_sync(tctx, tctx->sysdb, tctx->octx->name, tctx->octx);
+    ret = sysdb_getgrnam_sync(tctx, tctx->octx->name, tctx->octx);
     if (ret != EOK) {
         ERROR("Cannot find group in local domain, "
               "modifying groups is allowed only in local domain\n");
@@ -208,7 +208,7 @@ int main(int argc, const char **argv)
     in_transaction = true;
 
     /* groupmod */
-    tctx->error = groupmod(tctx, tctx->sysdb, tctx->octx);
+    tctx->error = groupmod(tctx, tctx->octx);
     if (tctx->error) {
         goto done;
     }
