@@ -31,7 +31,6 @@
 #include <ldb.h>
 #include <popt.h>
 #include <stdio.h>
-#include <signal.h>
 
 #include "util/util.h"
 #include "tools/common/sss_process.h"
@@ -255,7 +254,7 @@ errno_t sssctl_logs_remove(struct sss_cmdline *cmdline,
         sss_signal(SIGHUP);
     } else {
         PRINT("Truncating log files...\n");
-        ret = sssctl_run_command("truncate --size 0 " LOG_FILES);
+        ret = sssctl_run_command("truncate --no-create --size 0 " LOG_FILES);
         if (ret != EOK) {
             ERROR("Unable to truncate log files\n");
             return ret;
