@@ -35,6 +35,9 @@
 #define IN_BUF_SIZE         512
 #define CHILD_MSG_CHUNK     256
 
+#define SIGTERM_TO_SIGKILL_TIME 2
+#define CHILD_TIMEOUT_EXIT_CODE 7
+
 struct response {
     uint8_t *buf;
     size_t size;
@@ -101,21 +104,19 @@ int read_pipe_recv(struct tevent_req *req, TALLOC_CTX *mem_ctx,
 void fd_nonblocking(int fd);
 
 /* Never returns EOK, ether returns an error, or doesn't return on success */
-errno_t exec_child_ex(TALLOC_CTX *mem_ctx,
-                      int *pipefd_to_child, int *pipefd_from_child,
-                      const char *binary, int debug_fd,
-                      const char *extra_argv[], bool extra_args_only,
-                      int child_in_fd, int child_out_fd);
+void exec_child_ex(TALLOC_CTX *mem_ctx,
+                   int *pipefd_to_child, int *pipefd_from_child,
+                   const char *binary, const char *logfile,
+                   const char *extra_argv[], bool extra_args_only,
+                   int child_in_fd, int child_out_fd);
 
 /* Same as exec_child_ex() except child_in_fd is set to STDIN_FILENO and
  * child_out_fd is set to STDOUT_FILENO and extra_argv is always NULL.
  */
-errno_t exec_child(TALLOC_CTX *mem_ctx,
-                   int *pipefd_to_child, int *pipefd_from_child,
-                   const char *binary, int debug_fd);
+void exec_child(TALLOC_CTX *mem_ctx,
+                int *pipefd_to_child, int *pipefd_from_child,
+                const char *binary, const char *logfile);
 
 int child_io_destructor(void *ptr);
 
-errno_t child_debug_init(const char *logfile, int *debug_fd);
-
 #endif /* __CHILD_COMMON_H__ */
