@@ -234,7 +234,7 @@ errno_t diff_string_lists(TALLOC_CTX *memctx,
         list2 = _list2;
     }
 
-    error = hash_create(10, &table, NULL, NULL);
+    error = hash_create(0, &table, NULL, NULL);
     if (error != HASH_SUCCESS) {
         talloc_free(tmp_ctx);
         return EIO;
@@ -436,100 +436,6 @@ errno_t sss_hash_create(TALLOC_CTX *mem_ctx, unsigned long count,
     return sss_hash_create_ex(mem_ctx, count, tbl, 0, 0, 0, 0, NULL, NULL);
 }
 
-errno_t sss_filter_sanitize_ex(TALLOC_CTX *mem_ctx,
-                               const char *input,
-                               char **sanitized,
-                               const char *ignore)
-{
-    char *output;
-    size_t i = 0;
-    size_t j = 0;
-    char *allowed;
-
-    /* Assume the worst-case. We'll resize it later, once */
-    output = talloc_array(mem_ctx, char, strlen(input) * 3 + 1);
-    if (!output) {
-        return ENOMEM;
-    }
-
-    while (input[i]) {
-        /* Even though this character might have a special meaning, if it's
-         * explicitly allowed, just copy it and move on
-         */
-        if (ignore == NULL) {
-            allowed = NULL;
-        } else {
-            allowed = strchr(ignore, input[i]);
-        }
-        if (allowed) {
-            output[j++] = input[i++];
-            continue;
-        }
-
-        switch(input[i]) {
-        case '\t':
-            output[j++] = '\\';
-            output[j++] = '0';
-            output[j++] = '9';
-            break;
-        case ' ':
-            output[j++] = '\\';
-            output[j++] = '2';
-            output[j++] = '0';
-            break;
-        case '*':
-            output[j++] = '\\';
-            output[j++] = '2';
-            output[j++] = 'a';
-            break;
-        case '(':
-            output[j++] = '\\';
-            output[j++] = '2';
-            output[j++] = '8';
-            break;
-        case ')':
-            output[j++] = '\\';
-            output[j++] = '2';
-            output[j++] = '9';
-            break;
-        case '\\':
-            output[j++] = '\\';
-            output[j++] = '5';
-            output[j++] = 'c';
-            break;
-        case '\r':
-            output[j++] = '\\';
-            output[j++] = '0';
-            output[j++] = 'd';
-            break;
-        case '\n':
-            output[j++] = '\\';
-            output[j++] = '0';
-            output[j++] = 'a';
-            break;
-        default:
-            output[j++] = input[i];
-        }
-
-        i++;
-    }
-    output[j] = '\0';
-    *sanitized = talloc_realloc(mem_ctx, output, char, j+1);
-    if (!*sanitized) {
-        talloc_free(output);
-        return ENOMEM;
-    }
-
-    return EOK;
-}
-
-errno_t sss_filter_sanitize(TALLOC_CTX *mem_ctx,
-                            const char *input,
-                            char **sanitized)
-{
-    return sss_filter_sanitize_ex(mem_ctx, input, sanitized, NULL);
-}
-
 char *
 sss_escape_ip_address(TALLOC_CTX *mem_ctx, int family, const char *addr)
 {
@@ -618,6 +524,39 @@ errno_t add_string_to_list(TALLOC_CTX *mem_ctx, const char *string,
     return EOK;
 }
 
+errno_t del_string_from_list(const char *string,
+                             char ***list_p, bool case_sensitive)
+{
+    char **list;
+    int(*compare)(const char *s1, const char *s2);
+
+    if (string == NULL || list_p == NULL) {
+        DEBUG(SSSDBG_OP_FAILURE, "Missing string or list.\n");
+        return EINVAL;
+    }
+
+    if (!string_in_list(string, *list_p, case_sensitive)) {
+        return ENOENT;
+    }
+
+    compare = case_sensitive ? strcmp : strcasecmp;
+    list = *list_p;
+    int matches = 0;
+    int index = 0;
+    while (list[index]) {
+        if (compare(string, list[index]) == 0) {
+            matches++;
+            TALLOC_FREE(list[index]);
+        } else if (matches) {
+            list[index - matches] = list[index];
+            list[index] = NULL;
+        }
+        index++;
+    }
+
+    return EOK;
+}
+
 int domain_to_basedn(TALLOC_CTX *memctx, const char *domain, char **basedn)
 {
     const char *s;
@@ -1073,3 +1012,57 @@ int sss_rand(void)
     }
     return rand();
 }
+
+errno_t sss_canonicalize_ip_address(TALLOC_CTX *mem_ctx,
+                                    const char *address,
+                                    char **canonical_address)
+{
+    struct addrinfo hints;
+    struct addrinfo *result = NULL;
+    char buf[INET6_ADDRSTRLEN + 1];
+    int ret;
+
+    memset(&hints, 0, sizeof(struct addrinfo));
+    hints.ai_family = AF_UNSPEC;
+    hints.ai_flags = AI_NUMERICHOST;
+
+    ret = getaddrinfo(address, NULL, &hints, &result);
+    if (ret != 0) {
+        DEBUG(SSSDBG_OP_FAILURE, "Failed to canonicalize address [%s]: %s",
+              address, gai_strerror(ret));
+        return EINVAL;
+    }
+
+    ret = getnameinfo(result->ai_addr, result->ai_addrlen, buf, sizeof(buf),
+                      NULL, 0, NI_NUMERICHOST);
+    freeaddrinfo(result);
+    if (ret != 0) {
+        DEBUG(SSSDBG_OP_FAILURE, "Failed to canonicalize address [%s]: %s",
+              address, gai_strerror(ret));
+        return EINVAL;
+    }
+
+    *canonical_address = talloc_strdup(mem_ctx, buf);
+    if (*canonical_address == NULL) {
+        return ENOMEM;
+    }
+
+    return EOK;
+}
+
+/* According to the https://tools.ietf.org/html/rfc2181#section-11.
+ * practically no restrictions are imposed to a domain name per se.
+ *
+ * But since SSSD uses this name as a part of log file name,
+ * it is still required to avoid '/' as a safety measure.
+ */
+bool is_valid_domain_name(const char *domain)
+{
+    if (strchr(domain, '/') != NULL) {
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              "Forbidden symbol '/' in the domain name '%s'\n", domain);
+        return false;
+    }
+
+    return true;
+}
