@@ -56,6 +56,7 @@ enum sssd_errors {
     ERR_INVALID_DATA_TYPE,
     ERR_MISSING_DP_TARGET,
     ERR_ACCOUNT_UNKNOWN,
+    ERR_KRB5_PRINCIPAL_NOT_FOUND,
     ERR_INVALID_CRED_TYPE,
     ERR_NO_CREDS,
     ERR_CREDS_EXPIRED,
@@ -97,6 +98,7 @@ enum sssd_errors {
     ERR_REFERRAL,
     ERR_SELINUX_CONTEXT,
     ERR_SELINUX_NOT_MANAGED,
+    ERR_SELINUX_USER_NOT_FOUND,
     ERR_REGEX_NOMATCH,
     ERR_TIMESPEC_NOT_SUPPORTED,
     ERR_INVALID_CONFIG,
@@ -108,6 +110,7 @@ enum sssd_errors {
     ERR_IPA_GETKEYTAB_FAILED,
     ERR_TRUST_FOREST_UNKNOWN,
     ERR_P11_CHILD,
+    ERR_P11_CHILD_TIMEOUT,
     ERR_ADDR_FAMILY_NOT_SUPPORTED,
     ERR_SBUS_SENDER_BUS,
     ERR_SUBDOM_INACTIVE,
@@ -141,13 +144,33 @@ enum sssd_errors {
     ERR_GET_ACCT_DOM_CACHED,
     ERR_ID_OUTSIDE_RANGE,
     ERR_GID_DUPLICATED,
+    ERR_MULTIPLE_ENTRIES,
+
+    /* DBUS Errors */
+    ERR_SBUS_KILL_CONNECTION,
+    ERR_SBUS_EMPTY_STRING,
+    ERR_SBUS_CONNECTION_LIMIT,
+    ERR_SBUS_INVALID_STRING,
+    ERR_SBUS_INVALID_TYPE,
+    ERR_SBUS_UNKNOWN_SERVICE,
+    ERR_SBUS_UNKNOWN_INTERFACE,
+    ERR_SBUS_UNKNOWN_PROPERTY,
+    ERR_SBUS_UNKNOWN_OWNER,
+    ERR_SBUS_NO_REPLY,
+
+    /* ini parsing errors */
+    ERR_INI_OPEN_FAILED,
+    ERR_INI_INVALID_PERMISSION,
+    ERR_INI_PARSE_FAILED,
+    ERR_INI_ADD_SNIPPETS_FAILED,
+
     ERR_LAST            /* ALWAYS LAST */
 };
 
 #define SSSD_ERR_BASE(err) ((err) & ~ERR_MASK)
 #define SSSD_ERR_IDX(err) ((err) & ERR_MASK)
 #define IS_SSSD_ERROR(err) \
-    ((SSSD_ERR_BASE(err) == ERR_BASE) && ((err) <= ERR_LAST))
+    (((err) > 0) && (SSSD_ERR_BASE(err) == ERR_BASE) && ((err) <= ERR_LAST))
 
 #define ERR_OK      0
 /* Backwards compat */
@@ -164,4 +187,7 @@ enum sssd_errors {
  */
 const char *sss_strerror(errno_t error);
 
+/* return ldb error converted to an errno */
+errno_t sss_ldb_error_to_errno(int ldberr);
+
 #endif /* __SSSD_UTIL_ERRORS_H__ */
